package com.starbuds.server.service.message.dao;

import com.starbuds.server.common.demo.BookVO;
import com.starbuds.server.common.pojo.db.PrivateMsgDO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 私聊消息持久层
 *
 * @author gy
 * @date 2023/10/4
 */
@Mapper
@Repository
public interface PrivateMsgDAO {
    /**
     * 插入
     *
     * @param privateMsgDO
     * @return
     */
    int insert(PrivateMsgDO privateMsgDO);

    /**
     * 删除消息通过消息id
     *
     * @param id
     * @return
     */
    int deleteById(@Param("id") String id);

    /**
     * 更新消息状态
     *
     * @param id
     * @param state 0未发送 1已发送 10违规
     * @return
     */
    int updateState(@Param("id") String id,
                    @Param("state") int state);

    /**
     * 更新消息删除
     *
     * @param id
     * @param isDeleted 0未删除 1已删除
     * @return
     */
    int updateDeleted(@Param("id") String id,
                      @Param("isDeleted") int isDeleted);

    /**
     * 获取私聊消息列表
     *
     * @param targetId   目标id
     * @param state      0未发送 1已发送
     * @param isDeleted  0未删除 1已删除
     * @param pageCursor
     * @param pageSize
     * @return
     */
    List<PrivateMsgDO> selectList(@Param("targetId") long targetId,
                                  @Param("state") int state,
                                  @Param("isDeleted") int isDeleted,
                                  @Param("pageCursor") int pageCursor,
                                  @Param("pageSize") int pageSize);
}
