package com.starbuds.server.service.message.dao;

import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.UserDO;
import com.starbuds.server.common.pojo.message.SmsPushTaskDO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 短信推送任务 Mapper
 */
@Mapper
@Repository
public interface SmsPushTaskDAO {

    /**
     * 插入数据
     */
    int insert(SmsPushTaskDO po);

    /**
     * 插入批量数据
     */
    int insertBatch(List<SmsPushTaskDO> list);

    /**
     * 插入或更新数据
     */
    int insertOrUpdate(SmsPushTaskDO po);

    /**
     * 更新数据
     */
    int update(UpdateWrapper uw);

    /**
     * 查询一条记录
     */
    SmsPushTaskDO selectOne(QueryWrapper qw);

    /**
     * 查询所有记录
     */
    SmsPushTaskDO selectList(QueryWrapper qw);

    /**
     * 查询总数
     */
    int selectCount(QueryWrapper qw);

    /**
     * 查询列表(分页)
     */
    List<SmsPushTaskDO> selectPageList(QueryWrapper qw);

    /**
     * 按条件筛选发送短信的用户列表(分页)
     *
     * @param loginTimeStart
     * @param loginTimeEnd
     * @param rechargeStatus
     * @param pageCursor
     * @param pageSize
     * @return
     */
    List<UserDO> selectSendSmsUsers(@Param("loginTimeStart") Long loginTimeStart,
                                    @Param("loginTimeEnd") Long loginTimeEnd,
                                    @Param("rechargeStatus") Integer rechargeStatus,
                                    @Param("pageCursor") int pageCursor,
                                    @Param("pageSize") int pageSize);

    /**
     * 按条件筛选发送短信的用户列表数量
     *
     * @param loginTimeStart
     * @param loginTimeEnd
     * @param rechargeStatus
     * @return
     */
    int selectSendSmsUsersCount(@Param("loginTimeStart") Long loginTimeStart,
                                @Param("loginTimeEnd") Long loginTimeEnd,
                                @Param("rechargeStatus") Integer rechargeStatus);
}