package com.starbuds.server.service.message.factory;

import com.starbuds.server.common.constant.common.SelectMode;
import com.starbuds.server.common.constant.message.sms.ChannelConstant;
import org.springframework.stereotype.Component;

import java.util.Map;
import java.util.Random;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/2 11:16
 * @Description 短信渠道工厂
 */
@Component
public class SmsChannelFactory extends AbstractChannelFactory {

    /**
     * 产生短信渠道
     */
    @Override
    public String creatChannel(SelectMode mode) {
        String channel = null;
        switch (mode) {
            case Weight:
                channel = getChannelWithWeight();
                break;
            case Random:
                channel = getChannelWithRandom();
                break;
            case Round:
                channel = getChannelWithRound();
                break;
            default:
                channel = getChannelWithRound();
                break;
        }
        return channel;
    }

    /**
     * 权重
     */
    private String getChannelWithWeight() {
        //获取权重总和
        Integer sum = ChannelConstant.smsChannelWeightMap.values().parallelStream().reduce(Integer::sum).get();
        //随机数
        Integer random = new Random().nextInt(sum);
        for (Map.Entry<String, Integer> entry : ChannelConstant.smsChannelWeightMap.entrySet()) {
            String channel = entry.getKey();
            Integer weight = entry.getValue();
            if (random > entry.getValue()) {
                random -= weight;
            } else {
                return channel;
            }
        }
        return null;
    }

    /**
     * 随机
     */
    private String getChannelWithRandom() {
        //所有key
        String[] keys = ChannelConstant.smsChannelWeightMap.keySet().toArray(new String[0]);
        Random random = new Random();
        String randomKey = keys[random.nextInt(keys.length)];
        return randomKey;
    }

    /**
     * 轮询
     */
    private String getChannelWithRound() {
        //所有key
        String[] keys = ChannelConstant.smsChannelWeightMap.keySet().toArray(new String[0]);
        int totalSize = ChannelConstant.smsChannelWeightMap.size();
        int currentIndex = totalSize - 1;
        currentIndex = (currentIndex + 1) % totalSize;
        return keys[currentIndex];
    }
}
