package com.starbuds.server.service.message.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.lang.UUID;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.google.common.base.Preconditions;
import com.starbuds.server.common.constant.cache.RedisLiveKeys;
import com.starbuds.server.common.constant.cache.RedisUserKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.message.im.*;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.demo.BookVO;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.message.*;
import com.starbuds.server.common.pojo.message.im.BaseImMsg;
import com.starbuds.server.common.pojo.message.im.ImPushInfo;
import com.starbuds.server.common.pojo.message.im.biz.SystemMsg;
import com.starbuds.server.common.pojo.message.im.rtc.RtcAllRoomNoticeImMsg;
import com.starbuds.server.common.pojo.mq.QueueChatGroupMsg;
import com.starbuds.server.common.pojo.mq.QueueRtcUserExitMsg;
import com.starbuds.server.common.pojo.mq.QueueRtcUserJoinMsg;
import com.starbuds.server.common.pojo.user.UserBlackDTO;
import com.starbuds.server.common.pojo.user.UserSimpleInfo;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.message.dao.*;
import com.starbuds.server.service.message.entity.PrivateGameShareMsg;
import com.starbuds.server.service.message.entity.PrivateGiftMsg;
import com.starbuds.server.service.message.entity.PrivateUserFollowMsg;
import com.starbuds.server.service.message.provider.AgoraProvider;
import com.starbuds.server.service.message.provider.IImProvider;
import com.starbuds.server.service.message.provider.RongCloudProvider;
import com.starbuds.server.service.message.provider.TencentImProvider;
import com.starbuds.server.service.message.rocket.RocketProducer;
import com.starbuds.server.service.message.tencent.callback.TencentCallbackHandler;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

import static com.google.common.base.Preconditions.checkArgument;
import static com.starbuds.server.service.message.util.SpringContextUtil.isDev;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/11 16:42
 * @Description 网易云信服务
 */
@Slf4j
@Service(version = "1.0.0")
public class ImServiceImpl implements ImService {

    @Resource
    TencentCallbackHandler tencentCallbackHandler;

    // @Resource
    // TencentImProvider tencentImProvider;//腾讯im改为融云im

    // @Resource
    // AgoraProvider agoraProvider; //声网发送房间消息(之后也改为融云im)

    // @Resource
    // RongCloudProvider rongCloudProvider; //融云im Provider改为接口IImProvider

    // im功能调整为只用这一个=融云私聊、群聊、房间聊天室消息。如果同时使用要单独拎出来不用接口实现。或者@Resource(name = "xxx")
    @Resource(name = "rongCloudProvider")
    IImProvider imProvider;

    @Resource
    LiveMsgDAO liveMsgDAO;

    @Resource
    ChatMsgDAO chatMsgDAO;

    @Resource
    RtcMsgDAO rtcMsgDAO;

    @Resource
    UserNoSpeakHisDAO userNoSpeakHisDAO;

    @Resource
    PublicChatRoomMsgHisDAO publicChatRoomMsgHisDAO;

    @Resource
    RocketProducer rocketProducer;

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Resource
    PrivateMsgDAO privateMsgDAO;

    /**
     * 获取令牌
     *
     * @param imUserProfile
     * @return
     */
    @Override
    public String getToken(ImUserProfile imUserProfile) {
        return imProvider.buildToken(imUserProfile);
    }

    /**
     * 检查 SDK AppId
     *
     * @param sdkAppId
     * @return
     */
    @Override
    public boolean checkSdkAppId(Long sdkAppId) {
        return true;
    }

    /**
     * 处理腾讯 IM 回调
     *
     * @param callbackCommand
     * @param clientIp
     * @param optPlatform
     * @param content
     * @return
     */
    @Override
    public boolean handleTencentImCallback(String callbackCommand, String clientIp, String optPlatform, String content) {
        return tencentCallbackHandler.handleCallback(callbackCommand, clientIp, optPlatform, content);
    }

    /**
     * 同步用户资料
     *
     * @param imUserProfile
     * @return
     */
    @Override
    public boolean syncUserProfile(ImUserProfile imUserProfile) {
        return imProvider.refreshUser(imUserProfile);
    }

    /**
     * 发送自定义消息
     *
     * @param senderId
     * @param targetId
     * @param imMsg
     * @param imPushInfo
     * @return
     */
    @Override
    public boolean sendCustomImMsg(Long senderId, Long targetId, BaseImMsg imMsg, ImPushInfo imPushInfo) {
        if (ImMsgType.PRIVATE_CHAT_GIFT_SETTLE.equals(imMsg.getType())) {
            // 自定义消息类型
            PrivateGiftMsg msg = new PrivateGiftMsg(toJSON(imMsg));
            return imProvider.sendToPeerCustomMsg(senderId, targetId, msg, "收到新的礼物打赏，快来看看吧", BoolIntVal.True);
        } else if (ImMsgType.GAME_SHARE.equals(imMsg.getType())) {
            PrivateGameShareMsg msg = new PrivateGameShareMsg(toJSON(imMsg));
            return imProvider.sendToPeerCustomMsg(senderId, targetId, msg, "收到新的游戏邀请，快来看看吧", BoolIntVal.True);
        }
        return imProvider.sendToPeerCustomMsg(senderId, targetId, toJSON(imMsg));
    }

    /**
     * 发送没有跳转动作的系统消息
     *
     * @param targetUserId 接收消息的用户id
     * @param title
     * @param content
     * @return
     */
    @Override
    public boolean sendNoActionSystemMsg(long targetUserId, String title, String content) {
        checkArgument(StringUtils.isNotBlank(title), "title 不能为空");
        checkArgument(StringUtils.isNotBlank(content), "content 不能为空");

        SystemMsg systemMsg = new SystemMsg();
        systemMsg.setContentType(ContentType.TEXT);
        systemMsg.setTitle(title);
        systemMsg.setBody(content);
        systemMsg.setActionType(ImActionType.NO_ACTION.getValue());
        return sendCustomImMsg(null, targetUserId, systemMsg, null);
    }

    /**
     * 发送文本消息
     *
     * @param senderId
     * @param targetId
     * @param text
     * @return
     */
    @Override
    public boolean sendTextMsg(Long senderId, Long targetId, String text) {
        return imProvider.sendToPeerTextMsg(senderId, targetId, text, null);
    }

    /**
     * 发送直播间消息
     *
     * @param targetType
     * @param senderId
     * @param targetId
     * @param imMsg
     * @return
     */
    @Override
    public boolean sendLiveRoomMsg(Integer targetType, Long senderId, Long targetId, BaseImMsg imMsg) {
        try {
            senderId = Optional.ofNullable(senderId).orElse(0L);
            targetId = Optional.ofNullable(targetId).orElse(0L);
            long curTime = System.currentTimeMillis();
            String msgId = SecureUtil.md5(String.format("%s:%d:%d:%d:%d", imMsg.getType(), targetId, senderId, curTime, RandomUtil.randomInt(1000, 10000)));

            // 将消息写入数据库
            LiveMsgDO liveMsgDO = new LiveMsgDO();
            liveMsgDO.setId(msgId);
            liveMsgDO.setMsgType(imMsg.getType());
            liveMsgDO.setTargetType(targetType);
            liveMsgDO.setTargetId(targetId);
            liveMsgDO.setSenderId(senderId);
            liveMsgDO.setMsgData(toJSON(imMsg));
            liveMsgDO.setTimestamp(curTime);
            liveMsgDAO.insert(liveMsgDO);

            boolean lowPriority = imMsg.getType().equals(ImMsgType.LIVE_USER_JOIN);

            // 丢入消息队列
            if (ImTargetType.GROUP == targetType) {
                return imProvider.sendToChannel(senderId, targetId, liveMsgDO.getMsgData(), lowPriority);
            } else if (ImTargetType.USER == targetType) {
                return imProvider.sendToPeer(senderId, targetId, liveMsgDO.getMsgData());
            } else {
                return imProvider.sendToChannel(senderId, 0L, liveMsgDO.getMsgData(), lowPriority);
            }
        } catch (Exception e) {
            log.error("直播间消息投递失败", e);
        }
        return false;
    }

    /**
     * 发送 PK 组
     *
     * @param battleId
     * @param msg
     * @return
     */
    @Override
    public boolean sendBattleGroupMsg(String battleId, BaseImMsg msg) {
        try {
            Set<Object> keys = redisService.hashKeys(RedisLiveKeys.getLiveBattleValue(battleId));
            if (CollectionUtil.isNotEmpty(keys)) {
                for (Object key : keys) {
                    sendLiveRoomMsg(ImTargetType.GROUP, null, Convert.toLong(key), msg);
                }
            }
            return true;
        } catch (Exception e) {
            log.error("发送 PK 组消息失败", e);
        }
        return false;
    }

    /**
     * 发送聊天群组消息
     *
     * @param targetType
     * @param senderId
     * @param targetId
     * @param groupId
     * @param imMsg
     * @return
     */
    @Override
    public boolean sendChatGroupMsg(Integer targetType, Long senderId, Long targetId, String groupId, BaseImMsg imMsg) {
        try {
            senderId = Optional.ofNullable(senderId).orElse(0L);
            targetId = Optional.ofNullable(targetId).orElse(0L);
            groupId = Optional.ofNullable(groupId).orElse("");
            long curTime = System.currentTimeMillis();
            String msgId = SecureUtil.md5(String.format("%s:%s:%d:%d:%s", imMsg.getType(), groupId, senderId, curTime, UUID.fastUUID().toString(true)));

            // 将消息写入数据库
            ChatMsgDO chatMsgDO = new ChatMsgDO();
            chatMsgDO.setId(msgId);
            chatMsgDO.setMsgType(imMsg.getType());
            chatMsgDO.setTargetType(targetType);
            chatMsgDO.setGroupId(groupId);
            chatMsgDO.setSenderId(senderId);
            chatMsgDO.setTargetId(targetId);
            chatMsgDO.setMsgData(toJSON(imMsg));
            chatMsgDO.setTimestamp(curTime);
            chatMsgDAO.insert(chatMsgDO);

            // 丢入消息队列
            QueueChatGroupMsg chatGroupMsg = new QueueChatGroupMsg();
            BeanUtils.copyProperties(chatMsgDO, chatGroupMsg);
            rocketProducer.sendMsg(RocketTopic.TPC_LIVE, RocketTag.CHAT_GROUP, chatGroupMsg.serialize());

            return true;
        } catch (Exception e) {
            log.error("直播间消息投递失败", e);
        }
        return false;
    }

    /**
     * 发送im群组消息
     *
     * @param groupId
     * @param senderId
     * @param imMsg
     * @return
     */
    @Override
    public boolean sendImChatGroupMsg(String groupId, Long senderId, BaseImMsg imMsg) {
        return imProvider.sendToChatRoom(senderId, groupId, toJSON(imMsg));
    }

    /**
     * 发送 RTC 房间消息
     *
     * @param targetType
     * @param senderId
     * @param targetId
     * @param roomId
     * @param imMsg
     * @return
     */
    @Override
    public boolean sendRtcRoomMsg(Integer targetType, Long senderId, Long targetId, Long roomId, BaseImMsg imMsg) {
        try {
            senderId = Optional.ofNullable(senderId).orElse(0L);
            targetId = Optional.ofNullable(targetId).orElse(0L);
            roomId = Optional.ofNullable(roomId).orElse(0L);

            if (isDev()) {
                log.info("开始准备发送: imMsg:{}", JSON.toJSONString(imMsg));
            }
            long curTime = System.currentTimeMillis();
            String msgId = SecureUtil.md5(String.format("%s:%d:%d:%d:%s", imMsg.getType(), roomId, senderId, curTime, UUID.fastUUID().toString(true)));

            // 将消息写入数据库
            RtcMsgDO rtcMsgDO = new RtcMsgDO();
            rtcMsgDO.setId(msgId);
            rtcMsgDO.setMsgType(imMsg.getType());
            rtcMsgDO.setTargetType(targetType);
            rtcMsgDO.setRoomId(roomId);
            rtcMsgDO.setSenderId(senderId);
            rtcMsgDO.setTargetId(targetId);
            rtcMsgDO.setMsgData(toJSON(imMsg));
            rtcMsgDO.setTimestamp(curTime);
            rtcMsgDAO.insert(rtcMsgDO);
            log.info("消息写入数据库-完成");

            boolean lowPriority = imMsg.getType().equals(ImMsgType.LIVE_USER_JOIN);
            if (ImTargetType.GROUP == targetType) {
                return imProvider.sendToChannel(senderId, roomId, rtcMsgDO.getMsgData(), lowPriority);

            } else if (ImTargetType.USER == targetType) {
                return imProvider.sendToPeer(senderId, targetId, rtcMsgDO.getMsgData());

            } else {
                return imProvider.sendToChannel(senderId, 0L, rtcMsgDO.getMsgData(), lowPriority);
            }
        } catch (Exception e) {
            log.error("直播间消息投递失败", e);
        }
        return false;
    }

    /**
     * 发送全房间广播
     *
     * @param imMsg
     * @return
     */
    @Override
    public boolean sendAllRoomMsg(BaseImMsg imMsg) {
        return sendRtcRoomMsg(ImTargetType.ALL, null, null, null, imMsg);
    }

    /**
     * 发送全房间通知广播
     * 客户端显示"通知"开通是固定的，不能自定义有样式
     *
     * @param notice 通知内容
     * @return
     */
    @Override
    public boolean sendAllRoomNotice(String notice) {
        if (StringUtils.isBlank(notice)) {
            log.error("发送全房间通知广播 notice is blank");
            return false;
        }
        RtcAllRoomNoticeImMsg imMsg = new RtcAllRoomNoticeImMsg();
        imMsg.setNotice(notice);
        return sendAllRoomMsg(imMsg);
    }

    @Override
    public List<ImAppInfoDO> getAppInfo() {
        return Collections.emptyList();
    }

    /**
     * 添加黑名单
     *
     * @param userId
     * @param targetUserId
     * @return
     */
    @Override
    public boolean addBlack(Long userId, Long targetUserId) {
        return imProvider.addUserBlack(userId, targetUserId);
    }

    /**
     * 删除黑名单
     *
     * @param userId
     * @param targetUserId
     * @return
     */
    @Override
    public boolean delBlack(Long userId, Long targetUserId) {
        return imProvider.removeUserBlack(userId, targetUserId);
    }

    /**
     * 获取黑名单列表
     *
     * @param userId
     * @return
     */
    @Override
    public List<UserBlackDTO> getBlackList(Long userId, int pageIndex, int pageSize, Integer lastSequence) {
        List<UserBlackDTO> list = new ArrayList<>();
        List<String> userIds = imProvider.getUserBlack(userId);
        long curTime = System.currentTimeMillis();
        if (!CollectionUtils.isEmpty(userIds)) {
            for (String id : userIds) {
                UserBlackDTO userBlack = new UserBlackDTO();
                userBlack.setUserId(Convert.toLong(id));
                userBlack.setBlackTime(curTime);
                list.add(userBlack);
            }
        }
        return list;
    }

    /**
     * 获取用户在线状态
     *
     * @param userIds
     * @return
     */
    @Override
    public List<ImOnlineStatus> getImOnlineStatus(List<Long> userIds) {
        return Collections.emptyList();
    }

    /**
     * 获得房间内在线用户id（最多返回 500 个成员信息）
     *
     * @param roomId
     * @return 房间内在线用户id
     * @throws Exception
     */
    @Override
    public List<Long> onlineUserIdsInRoom(Long roomId) throws Exception {
        return imProvider.onlineUserIdsInRoom(roomId);
    }

    /**
     * 获取公聊大厅消息历史列表
     *
     * @param groupId
     * @param pageSize
     * @param lastMsgId
     * @return
     */
    @Override
    public List<PublicChatRoomMsgHisVO> getPublicChatRoomMsgHisList(String groupId, int pageSize, Integer lastMsgId) {
        return publicChatRoomMsgHisDAO.selectHisList(groupId, pageSize, lastMsgId);
    }

    /**
     * 禁言
     *
     * @param userId
     * @param privateChatDuration 私聊禁言时间
     * @param groupChatDuration   群聊禁言时间
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean setUserNoSpeaking(Long userId, Integer privateChatDuration, Integer groupChatDuration) {
        UserNoSpeakHisDO hisDO = new UserNoSpeakHisDO();
        hisDO.setHisId(identityService.getId());
        hisDO.setAdminId(0L);
        hisDO.setAdminName("系统调用");
        hisDO.setDuration(privateChatDuration.longValue());

        hisDO.setTriggerType(privateChatDuration > 0 ? BoolIntVal.True : BoolIntVal.False);
        hisDO.setUserId(userId);
        hisDO.setReason("系统调用封禁");
        Long now = System.currentTimeMillis();
        hisDO.setCreateTime(now);
        hisDO.setExpireTime(now + privateChatDuration);
        if (privateChatDuration == Integer.MAX_VALUE) {
            hisDO.setIsHandle(BoolIntVal.True);
        } else {
            hisDO.setIsHandle(BoolIntVal.False);
        }
        userNoSpeakHisDAO.insert(hisDO);
        boolean res = triggerForbidden(userId, privateChatDuration);
        return res;
    }

    /**
     * 取消禁封
     *
     * @param userId
     * @return
     */
    @Override
    public boolean cancelForbiddenWords(Long userId) {
        return imProvider.cancelForbiddenWords(userId);
    }

    private boolean triggerForbidden(Long userId, Integer privateChatDuration) {
        if (privateChatDuration != null) {
            if (privateChatDuration > 0) {
                return imProvider.forbiddenWords(userId);
            } else {
                return imProvider.cancelForbiddenWords(userId);
            }
        }
        return true;
    }

    /**
     * 用户禁言
     *
     * @param userId
     * @param privateChatDuration
     * @param reason
     * @param adminId
     * @param adminName
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean setUserNoSpeaking(Long userId, Integer privateChatDuration, String reason, Long adminId, String adminName) {
        UserNoSpeakHisDO hisDO = new UserNoSpeakHisDO();
        hisDO.setHisId(identityService.getId());
        hisDO.setAdminId(adminId);
        hisDO.setAdminName(adminName);
        hisDO.setDuration(privateChatDuration.longValue());
        hisDO.setCreateTime(System.currentTimeMillis());
        hisDO.setTriggerType(privateChatDuration > 0 ? BoolIntVal.True : BoolIntVal.False);
        hisDO.setUserId(userId);
        hisDO.setReason(reason);
        Long now = System.currentTimeMillis();
        hisDO.setCreateTime(now);
        hisDO.setExpireTime(now + privateChatDuration);
        if (privateChatDuration == Integer.MAX_VALUE || privateChatDuration == 0) {
            hisDO.setIsHandle(BoolIntVal.True);
        } else {
            hisDO.setIsHandle(BoolIntVal.False);
        }
        userNoSpeakHisDAO.insert(hisDO);
        boolean res = triggerForbidden(userId, privateChatDuration);
        return res;
    }

    /**
     * 查询用户禁言
     *
     * @param userId
     * @return
     */
    @Override
    public ImNoSpeaking getUserNoSpeaking(Long userId) {
        ImNoSpeaking imNoSpeaking = new ImNoSpeaking();
        imNoSpeaking.setPrivateChatDuration(0);
        imNoSpeaking.setGroupChatDuration(0);
        return imNoSpeaking;
    }

    /**
     * 获取声网 RTM 令牌
     *
     * @param userId
     * @return
     */
    @Override
    public String getAgoraRtmToken(Long userId) {
        return imProvider.buildToken(userId);
        // return agoraProvider.buildToken(userId);
    }

    /**
     * 获取融云令牌
     *
     * @param userId
     * @return
     */
    @Override
    public String getRongCloudToken(Long userId) {
        return imProvider.buildToken(userId);
    }

    /**
     * 融云验签
     *
     * @param timestamp
     * @param nonce
     * @param signature
     * @return
     */
    @Override
    public boolean checkRongCloudSign(String timestamp, String nonce, String signature) {
        return imProvider.checkNotifySign(timestamp, nonce, signature);
    }

    /**
     * 融云房间状态同步通知处理
     *
     * @param body
     */
    @Override
    public void handleRongCloudRoomStatusNotify(String body) {
        List<RongCloudRoomStatusNotify> originList = JSON.parseArray(body, RongCloudRoomStatusNotify.class);
        List<RongCloudRoomStatusNotify> list = originList.stream().sorted(Comparator.comparing(RongCloudRoomStatusNotify::getTime)).collect(Collectors.toList());
        for (RongCloudRoomStatusNotify notify : list) {
            if (!NumberUtil.isLong(notify.getChatRoomId())) {
                continue;
            }

            // 过滤掉无效房间
            // 0 作为公共全频道
            // 负数作为特殊功能固定频道
            Long roomId = Convert.toLong(notify.getChatRoomId());
            if (roomId <= 0) {
                continue;
            }

            if (RongCloudRoomStatusNotifyType.USER_JOIN == notify.getType()) {
                // 用户加入房间
                for (String userIdStr : notify.getUserIds()) {
                    Long userId = Convert.toLong(userIdStr);
                    String appVersion = redisService.get(RedisUserKeys.getUserAppVer(userId));
                    // 排查Bug：用户在房间内，在线列表中看不到
                    log.info("融云-rong-userJoin-{}-{}", userId, appVersion);
                    if (isDev()) {
                        log.info(LogUtil.devLog("融云回调用户[进入]房间, 房间id={} userId={} appVersion={}"),
                                roomId, userId, appVersion);
                    }

                    QueueRtcUserJoinMsg userJoinMsg = new QueueRtcUserJoinMsg();
                    userJoinMsg.setJoinUserId(userId);
                    userJoinMsg.setRoomId(roomId);
                    userJoinMsg.setTimestamp(notify.getTime());
                    userJoinMsg.setChannelId(0L);
                    userJoinMsg.setAppVer(appVersion);
                    // 排查Bug：用户在房间内，在线列表中看不到
                    log.info("融云-rong-userJoin-{}-{}-{}-sendMsg", userId, roomId, appVersion);
                    sendEnterRoomMsg(userJoinMsg);
                }
            } else if (RongCloudRoomStatusNotifyType.USER_EXIT == notify.getType()) {
                // 用户退出房间
                for (String userId : notify.getUserIds()) {
                    if (isDev()) {
                        log.info(LogUtil.devLog("融云回调用户[退出]房间, 房间id={} userId={}"), roomId, userId);
                    }
                    QueueRtcUserExitMsg userExitMsg = new QueueRtcUserExitMsg();
                    userExitMsg.setExitUserId(Convert.toLong(userId));
                    userExitMsg.setRoomId(roomId);
                    userExitMsg.setTimestamp(notify.getTime());
                    userExitMsg.setChannelId(0L);
                    rocketProducer.sendMsgOneway(RocketTopic.TPC_NORMAL, RocketTag.RTC_USER_EXIT, userExitMsg.serialize());
                }
            }
        }
    }

    @Override
    public void sendEnterRoomMsg(QueueRtcUserJoinMsg userJoinMsg) {
        rocketProducer.sendMsgOneway(RocketTopic.TPC_NORMAL, RocketTag.RTC_USER_JOIN, userJoinMsg.serialize());
    }

    /**
     * 融云用户状态同步通知处理
     *
     * @param body
     */
    @Override
    public void handleRongCloudUserStatusNotify(String body) {
        List<RongCloudUserStatusNotify> list = JSON.parseArray(body, RongCloudUserStatusNotify.class);
        for (RongCloudUserStatusNotify notify : list) {
            // 排查Bug：用户在房间内，在线列表中看不到
            log.info("rong-userStatusNotify-{}-{}-receive", notify.getUserid(), "0".equals(notify.getStatus()) ? BoolIntVal.True : BoolIntVal.False);
            userService.setIsOnline(Convert.toLong(notify.getUserid()), "0".equals(notify.getStatus()) ? BoolIntVal.True : BoolIntVal.False, notify.getClientIp(), notify.getOs());
        }
    }

    /**
     * 禁言
     *
     * @param roomId
     * @param userId
     * @param minutes
     * @return
     */
    @Override
    public boolean forbiddenChat(Long roomId, Long userId, Integer minutes) {
        return imProvider.forbiddenChat(roomId, userId, minutes);
    }

    /**
     * 解除禁言
     *
     * @param roomId
     * @param userId
     * @return
     */
    @Override
    public boolean enableChat(Long roomId, Long userId) {
        return imProvider.enableChat(roomId, userId);
    }

    /**
     * 房间拉黑
     *
     * @param roomId
     * @param userId
     * @param minutes
     * @return
     */
    @Override
    public boolean blockRoomUser(Long roomId, Long userId, Integer minutes) {
        return imProvider.blockRoomUser(roomId, userId, minutes);
    }

    /**
     * 解除拉黑
     *
     * @param roomId
     * @param userId
     * @return
     */
    @Override
    public boolean unblockRoomUser(Long roomId, Long userId) {
        return imProvider.unblockRoomUser(roomId, userId);
    }

    /**
     * 设置房间属性
     *
     * @param roomId
     * @param key
     * @param value
     * @return
     */
    @Override
    public boolean setRoomProperty(Long roomId, String key, String value) {
        return imProvider.setRoomProperty(roomId, key, value);
    }

    /**
     * 批量设置房间属性
     *
     * @param roomId
     * @param propertyMap
     */
    @Override
    public void setRoomProperties(Long roomId, Map<String, String> propertyMap) {
        imProvider.setRoomProperties(roomId, propertyMap);
    }

    /**
     * 获取 IM 用户属性
     *
     * @param userId
     * @return
     */
    @Override
    public UserSimpleInfo getUserInfo(Long userId) {
        return imProvider.getUserInfo(userId);
    }

    /**
     * 获取历史消息下载地址
     *
     * @param date
     * @return
     */
    @Override
    public String getHistoryUrl(String date) {
        return imProvider.getHistory(date);
    }

    private static String toJSON(Object obj) {
        return JSON.toJSONString(obj, SerializerFeature.WriteNonStringKeyAsString, SerializerFeature.DisableCircularReferenceDetect);
    }

    // ---im-server的私聊---

    /**
     * 存储私聊消息
     *
     * @param targetType
     * @param state 0未发送 1已发送
     * @param senderId
     * @param targetUserId
     * @param msgType
     * @param msgData
     * @return
     */
    @Override
    public boolean addPrivateMsg(String msgId, int targetType, int state, long senderId, long targetUserId, String msgType, String msgData){
        long curTime = System.currentTimeMillis();
        //String msgId = SecureUtil.md5(String.format("%s:%d:%d:%d:%s", msgType, targetUserId, senderId, curTime, UUID.fastUUID().toString(true)));
        //前端发起的消息msgid改为前端自己创建，好做消息的违规提示、对消息本身的处理

        PrivateMsgDO privateMsgDO = new PrivateMsgDO();
        privateMsgDO.setId(msgId);
        privateMsgDO.setState(state);
        privateMsgDO.setTargetType(targetType);
        privateMsgDO.setSenderId(senderId);
        privateMsgDO.setTargetId(targetUserId);
        privateMsgDO.setMsgType(msgType);
        privateMsgDO.setMsgData(msgData);
        privateMsgDO.setCreateTime(curTime);
        privateMsgDO.setUpdateTime(curTime);
        privateMsgDO.setIsDeleted(0);
        privateMsgDAO.insert(privateMsgDO);
        return true;
    }

    /**
     * 获取私聊消息
     *
     * @param targetId
     * @param state
     * @param isDeleted
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @Override
    public PageList<PrivateMsgDO> getPrivateMsgList(Long targetId, int state, int isDeleted, int pageIndex, int pageSize){
        PageList<PrivateMsgDO> pageList = new PageList<>(pageIndex, pageSize);
        List<PrivateMsgDO> list = privateMsgDAO.selectList(targetId, state, isDeleted, pageList.getPageCursor(), pageSize);
        pageList.setList(list);
        pageList.setTotal(list.size());
        return pageList;
    }

    /**
     * 设置私聊状态
     *
     * @param msgId
     * @param state
     * @return
     */
    @Override
    public boolean setMsgState(String msgId, Integer state){
        privateMsgDAO.updateState(msgId, state);
        return true;
    }

}
