package com.starbuds.server.service.message.impl;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.pojo.db.PushDeviceDO;
import com.starbuds.server.common.pojo.message.push.PushMsg;
import com.starbuds.server.common.service.message.PushService;
import com.starbuds.server.service.message.dao.PushDeviceDAO;
import com.starbuds.server.service.message.provider.AliyunPushProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.scheduling.annotation.Async;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/2 17:36
 * @Description 推送服务业务实现
 */
@Slf4j
@Service(version = "1.0.0")
public class PushServiceImpl implements PushService {

    @Resource
    AliyunPushProvider aliyunPushProvider;

    @Resource
    PushDeviceDAO pushDeviceDAO;

    @Override
    public boolean pushMsg(PushMsg pushMsg, String packageName) {
        log.info("准备推送消息pushMsg：{},packageName：{}", JSON.toJSONString(pushMsg), packageName);
        if (checkParams(pushMsg, packageName)) {
            push(pushMsg, packageName);
            return true;
        }
        log.info("参数校验不通过");
        return false;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean registerDevice(PushDeviceDO deviceDO) {
        return pushDeviceDAO.insertOrUpdate(deviceDO) > 0;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int removeDevice(String deviceId) {
        if (StringUtils.isEmpty(deviceId)) {
            return 0;
        }
        return pushDeviceDAO.deleteByDeviceId(deviceId);
    }

    /**
     * 推送
     *
     * @param pushMsg
     * @param packageName
     * @return
     */
    @Async
    public void push(PushMsg pushMsg, String packageName) {
        aliyunPushProvider.push(pushMsg, packageName);
    }

    /**
     * 校验参数
     *
     * @param pushMsg
     * @param packageName
     * @return
     */
    private boolean checkParams(PushMsg pushMsg, String packageName) {
        if (StringUtils.isEmpty(packageName)) {
            log.info("包名为空");
            return false;
        }

        if (pushMsg.getPushTarget() == null) {
            log.info("推送目标为空");
            return false;
        }

        if (pushMsg.getDeviceType() == null) {
            log.info("推送设备类型为空");
            return false;
        }
        return true;
    }

}
