package com.starbuds.server.service.message.impl;

import com.starbuds.server.common.service.message.SmsProviderService;
import com.starbuds.server.common.service.message.SmsPushService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Service;

import java.util.HashMap;
import java.util.Map;

/***
 * @author qinjiang
 * @description 短信发送服务中介
 * @date 2021/9/18
 */
@Slf4j
@Service(version = "1.0.0")
public class SmsProvider implements SmsProviderService {
    private Map<String, SmsPushService> providerMap = new HashMap<>();

    /**
     * 注册服务类
     *
     * @param smsAgentName
     * @param serviceImpl
     */
    @Override
    public void register(String smsAgentName, SmsPushService serviceImpl) {
        log.info("--SmsProvider smsAgentName");
        providerMap.put(smsAgentName, serviceImpl);
    }

    @Override
    public Boolean existInstance(String smsAgentName) {
        return providerMap.get(smsAgentName) != null;
    }

    @Override
    public Integer maxQps(String smsAgentName) {
        return this.getInstance(smsAgentName).maxQps();
    }

    @Override
    public Boolean send(String smsAgentName, String tel, Map<String, String> varMap, String templateId, String accessKey, String accessSecret, String sign) {
        return this.getInstance(smsAgentName).send(tel, varMap, templateId, accessKey, accessSecret, sign);
    }

    /**
     * 通过别名获得实体
     *
     * @param smsAgentName
     */
    public SmsPushService getInstance(String smsAgentName) {
        log.info("SmsProvider:instance:providerMap:" + providerMap);
        log.info("SmsProvider:instance:smsAgentName:" + smsAgentName);
        SmsPushService instance = providerMap.get(smsAgentName);
        log.info("--SmsProviderService :" + instance);
        return instance;
    }
}
