package com.starbuds.server.service.message.impl;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.SmsSignDO;
import com.starbuds.server.common.pojo.db.SmsTemplateDO;
import com.starbuds.server.common.pojo.db.UserDO;
import com.starbuds.server.common.pojo.message.SmsPushTaskDO;
import com.starbuds.server.common.pojo.message.bo.SendSmsUserBO;
import com.starbuds.server.common.pojo.message.qo.SmsPushTaskPageListQO;
import com.starbuds.server.common.pojo.message.qo.SmsPushTaskSendUsersPageListQO;
import com.starbuds.server.common.pojo.message.sms.SmsPushTaskMatchCondition;
import com.starbuds.server.common.pojo.message.vo.SmsPushTaskVO;
import com.starbuds.server.common.service.message.SmsPushTaskService;
import com.starbuds.server.service.message.dao.SmsPushTaskDAO;
import com.starbuds.server.service.message.dao.SmsSignDAO;
import com.starbuds.server.service.message.dao.SmsTemplateDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

/**
 * 短信推送任务
 *
 * @author aoe
 * @date 2021/7/5
 */
@Slf4j
@Service(version = "1.0.0")
public class SmsPushTaskServiceImpl implements SmsPushTaskService {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "smsPushTaskId";

    @Resource
    private SmsPushTaskDAO smsPushTaskDAO;

    @Resource
    private SmsSignDAO smsSignDAO;

    @Resource
    private SmsTemplateDAO smsTemplateDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public SmsPushTaskDO create(SmsPushTaskDO po) {
        po.setCreateTime(System.currentTimeMillis());
        po.setUpdateTime(po.getCreateTime());
        boolean flag = smsPushTaskDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("创建成功 {}", json);
            return po;
        }
        log.warn("创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean createBatch(List<SmsPushTaskDO> list) {
        boolean flag = smsPushTaskDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean createOrUpdate(SmsPushTaskDO po) {
        boolean flag = smsPushTaskDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateById(SmsPushTaskDO po) {
        po.setCreateTime(null);
        po.setUpdateTime(System.currentTimeMillis());

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getSmsPushTaskId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(UpdateWrapper uw) {
        boolean flag = smsPushTaskDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    @Override
    public SmsPushTaskDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return smsPushTaskDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    @Override
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return smsPushTaskDAO.selectCount(qw) > 0;
    }

    /**
     * 分页查询
     */
    @Override
    public PageList<SmsPushTaskDO> getPageList(SmsPushTaskPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<SmsPushTaskDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(smsPushTaskDAO.selectPageList(qw));
        page.setTotal(smsPushTaskDAO.selectCount(qw));
        return page;
    }

    // 手动编码开始

    /**
     * 分页查询 PageList<SmsPushTaskDO> 转 PageList<SmsPushTaskVO>
     *
     * @param pageList
     * @return
     */
    @Override
    public PageList<SmsPushTaskVO> toVoPageList(PageList<SmsPushTaskDO> pageList) {
        PageList<SmsPushTaskVO> pl = new PageList<>();
        BeanUtils.copyProperties(pageList, pl);
        List<SmsPushTaskDO> list = pageList.getList();
        if (CollectionUtils.isEmpty(list)) {
            return pl;
        }

        Map<Long, SmsSignDO> smsSignMap = smsSignDAO.selectSmsSignMap();
        Map<Long, SmsTemplateDO> templateMap = smsTemplateDAO.selectTemplateMap();

        List<SmsPushTaskVO> result = list.stream()
                .map(task -> {
                    SmsPushTaskVO vo = new SmsPushTaskVO();
                    BeanUtils.copyProperties(task, vo);

                    String matchConditionStr = task.getMatchCondition();
                    if (JSON.isValid(matchConditionStr)) {
                        SmsPushTaskMatchCondition matchCondition = JSON.parseObject(matchConditionStr, SmsPushTaskMatchCondition.class);
                        if (matchCondition != null) {
                            BeanUtils.copyProperties(matchCondition, vo);
                        }
                    }

                    SmsSignDO sign = smsSignMap.get(vo.getSignatureId());
                    if (sign != null) {
                        vo.setSignName(sign.getSignName());
                    }

                    SmsTemplateDO template = templateMap.get(vo.getTempId());
                    if (template != null) {
                        vo.setTempCode(template.getTemplateCode());
                        vo.setAgentId(template.getAgentId());
                    }
                    return vo;
                })
                .collect(Collectors.toList());
        pl.setList(result);

        return pl;
    }

    /**
     * 短信推送任务用户信息分页查询
     *
     * @param qo
     * @return
     */
    @Override
    public PageList<SendSmsUserBO> getSendSmsUserPageList(SmsPushTaskSendUsersPageListQO qo) {
        PageList<SendSmsUserBO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        page.setList(getSendSmsUsers(qo));
        page.setTotal(getSendSmsUsersCount(qo));
        return page;
    }

    @Override
    public Integer getRechargeStatus(Integer rechargeStatus) {
        if (rechargeStatus == 1) {
            return BoolIntVal.True;
        } else if (rechargeStatus == 2) {
            return BoolIntVal.False;
        }
        return null;
    }

    /**
     * 按条件筛选发送短信的用户列表
     *
     * @param qo
     * @return
     */
    private List<SendSmsUserBO> getSendSmsUsers(SmsPushTaskSendUsersPageListQO qo) {
        PageList<SendSmsUserBO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        List<UserDO> list = smsPushTaskDAO.selectSendSmsUsers(qo.getLoginTimeStart(), qo.getLoginTimeEnd(), qo.getRechargeStatus(),
                page.getPageCursor(), page.getPageSize());
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyList();
        }
        return list.stream()
                .map(user -> {
                    SendSmsUserBO bo = new SendSmsUserBO();
                    BeanUtils.copyProperties(user, bo);
                    return bo;
                })
                .collect(Collectors.toList());
    }

    /**
     * 按条件筛选发送短信的用户列表数量
     *
     * @param qo
     * @return
     */
    private int getSendSmsUsersCount(SmsPushTaskSendUsersPageListQO qo) {
        return smsPushTaskDAO.selectSendSmsUsersCount(qo.getLoginTimeStart(), qo.getLoginTimeEnd(), qo.getRechargeStatus());
    }
}
