package com.starbuds.server.service.message.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.RandomUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.constant.cache.RedisCacheKeys;
import com.starbuds.server.common.constant.common.AppEnv;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.common.TimeConstants;
import com.starbuds.server.common.constant.message.sms.SmsActionEnum;
import com.starbuds.server.common.constant.message.sms.SmsAgentNameConstant;
import com.starbuds.server.common.constant.message.sms.SmsTemplateTypeEnum;
import com.starbuds.server.common.constant.message.sms.SmsTypeEnum;
import com.starbuds.server.common.pojo.data.SmsAgentKeySecretName;
import com.starbuds.server.common.pojo.db.SmsCodeHisDO;
import com.starbuds.server.common.pojo.db.SmsSignDO;
import com.starbuds.server.common.pojo.message.SendResult;
import com.starbuds.server.common.pojo.message.SmsCode;
import com.starbuds.server.common.pojo.message.VoiceSmsCode;
import com.starbuds.server.common.pojo.user.ChannelSmsConfig;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.data.SmsConfigService;
import com.starbuds.server.common.service.message.SmsService;
import com.starbuds.server.common.service.statistics.StatPlatformService;
import com.starbuds.server.common.service.user.ChannelService;
import com.starbuds.server.common.service.user.SecurityService;
import com.starbuds.server.service.message.constant.StringConstant;
import com.starbuds.server.service.message.dao.SmsCodeHisDAO;
import com.starbuds.server.service.message.provider.AliyunSmsProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

import static com.starbuds.server.service.message.util.SpringContextUtil.isDev;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/2 10:33
 * @Description 短信接口实现
 */
@Slf4j
@Service(version = "1.0.0")
public class SmsServiceImpl implements SmsService {

    private static final long smsExpireMill = 10 * 60 * 1000;

    public static final String showNumber = "055162143345";

    @Resource
    AliyunSmsProvider aliyunSmsProvider;

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Resource
    private SmsCodeHisDAO smsCodeHisDAO;

    @Value("${aliyun.push.env}")
    String pushEnv;

    @Reference(version = "1.0.0")
    SmsConfigService smsConfigService;

    @Reference(version = "1.0.0")
    ChannelService channelService;

    @Reference(version = "1.0.0")
    SecurityService securityService;

    @Reference(version = "1.0.0")
    StatPlatformService statPlatformService;

    /**
     * 发送短信验证码
     *
     * @param action    短信动作
     * @param phone     手机号
     * @param isDevSend 是否开发环境
     * @param channelId 渠道id
     * @param areaCode  区号
     * @return 标识符
     */
    @Override
    public SendResult sendSmsCode(SmsActionEnum action, String phone, boolean isDevSend, Long channelId, String areaCode) {
        return sendSmsCode(action, phone, isDevSend, channelId, areaCode, null, null);
    }

    /**
     * 发送短信验证码
     *
     * @param action
     * @param phone
     * @param isDevSend
     * @param channelId
     * @param areaCode
     * @param ip
     * @param deviceId
     * @return
     */
    @Override
    public SendResult sendSmsCode(SmsActionEnum action, String phone, boolean isDevSend, Long channelId, String areaCode, String ip, String deviceId) {
        log.info("channelId:{},areaCode:{},phone:{}", channelId, areaCode, phone);
        SendResult sendResult = new SendResult();

        // 判断是否发送过于频繁
        if (!check(action, phone)) {
            sendResult.setMsg("短信验证码发送过于频繁，请稍后再试");
            return sendResult;
        }

        // 获取渠道短信配置
        ChannelSmsConfig smsConfig = channelService.getSmsConfig(channelId);
        if (smsConfig == null || smsConfig.getSmsAgentId() == null) {
            sendResult.setMsg("未找到渠道短信配置");
            return sendResult;
        }

        // 获取短信配置
        SmsAgentKeySecretName agent = smsConfigService.getAgentKeyAndSecret(smsConfig.getSmsAgentId());
        SmsSignDO signDO = smsConfigService.getSign(smsConfig.getSmsSignId(), smsConfig.getSmsAgentId());
        String templateCode = smsConfigService.getTemplateCode(smsConfig.getSmsAgentId(), SmsTypeEnum.CODE, action, "86".equals(areaCode) ? SmsTemplateTypeEnum.CHINA : SmsTemplateTypeEnum.INTERNAL);
        if (agent == null) {
            sendResult.setMsg("未找到短信服务商配置");
            return sendResult;
        }

        String agentName = agent.getAgentName();
        // 验证码
        String verifyCode = RandomUtil.randomNumbers(4);

        boolean isDev = AppEnv.DEV.equals(pushEnv);
        String verifyPhone = phone;
        if (phone.startsWith(areaCode)) {
            verifyPhone = phone.substring(areaCode.length());
        }
        if (securityService.isMarketAuditorPhone(verifyPhone, areaCode)) {
            // 市场审核人员账号不发验证码 直接返回
            isDev = true;
            isDevSend = false;
        }
        // 判断是否需要真实发送，只有正式环境 或者 测试环境并传真实发送参数才发送
        if (!isDev || isDevSend) {
            if ("aliyun".equals(agentName)) {
                String signName = signDO.getSignName();
                if (!"86".equals(areaCode)) {
                    signName = signDO.getInternalSignName();
                }

                log.info("templateCode:{},phone:{},verifyCode:{},signName:{}", templateCode, phone, verifyCode, signName);

                // todo 开始请求第三方发送验证码
                String result = aliyunSmsProvider.sendSms(templateCode, phone, new SmsCode(verifyCode), agent.getAccessKey(), agent.getAccessSecret(), signName);

                log.info("result:{}", result);
                if (StringUtils.isBlank(result)) {
                    sendResult.setMsg("短信服务商异常");
                    return sendResult;
                }

                // 从json结果读msg
                String message = JSON.parseObject(result).getString("Message");
                if (!StringConstant.OK.equalsIgnoreCase(message)) {
                    sendResult.setMsg(message);
                    return sendResult;
                }
            } else if (SmsAgentNameConstant.BAIDU.equals(agentName)) {

            } else {
                sendResult.setMsg("未开通该服务商的短信功能");
                return sendResult;
            }
        } else {
            // 测试环境模拟延迟
            try {
                Thread.sleep(200);
            } catch (InterruptedException e) {
                log.error("测试环境 sleep 中断", e);
            }
        }

        // 写入短信记录
        writeHis(action, phone, verifyCode, smsConfig.getSmsAgentId(), channelId, ip, deviceId);

        // 发送成功
        sendResult.setSuccess(true);
        // 测试环境或者审核人员账号需要回填验证码
        if (isDev) {
            sendResult.setData(verifyCode);
        }

        return sendResult;
    }

    @Override
    public String sendSmsCodeOnlyTest(SmsActionEnum action, String phone, Long channelId, String areaCode, String ip, String deviceId) {
        if (!isDev()) {
            return null;
        }
        // 验证码
        String verifyCode = RandomUtil.randomNumbers(4);
        ChannelSmsConfig smsConfig = channelService.getSmsConfig(channelId);
        // 写入短信记录
        writeHis(action, phone, verifyCode, smsConfig.getSmsAgentId(), channelId, ip, deviceId);
        return verifyCode;
    }

    /**
     * 发送短信验证码
     *
     * @param action
     * @param phone
     * @param isDevSend
     * @param channelId
     * @param areaCode
     * @param ip
     * @param deviceId
     * @return
     */
    @Override
    public SendResult sendVoiceCode(SmsActionEnum action, String phone, boolean isDevSend, Long channelId, String areaCode, String ip, String deviceId) {
        log.info("channelId:{},areaCode:{},phone:{}", channelId, areaCode, phone);
        SendResult sendResult = new SendResult();

        // 判断是否发送过于频繁
        if (!check(action, phone)) {
            sendResult.setMsg("短信验证码发送过于频繁，请稍后再试");
            return sendResult;
        }

        // 获取渠道短信配置
        ChannelSmsConfig smsConfig = channelService.getSmsConfig(channelId);
        if (smsConfig == null || smsConfig.getSmsAgentId() == null) {
            sendResult.setMsg("未找到渠道短信配置");
            return sendResult;
        }

        // 获取短信配置
        SmsAgentKeySecretName agent = smsConfigService.getAgentKeyAndSecret(smsConfig.getSmsAgentId());
        log.info("短信服务商:{}", JSON.toJSONString(agent));
        SmsSignDO signDO = smsConfigService.getSign(smsConfig.getSmsSignId(), smsConfig.getSmsAgentId());
        log.info("签名:{}", JSON.toJSONString(signDO));
        String templateCode = smsConfigService.getTemplateCode(smsConfig.getSmsAgentId(), SmsTypeEnum.CODE, action, "86".equals(areaCode) ? SmsTemplateTypeEnum.CHINA : SmsTemplateTypeEnum.INTERNAL);
        log.info("模板:{}", templateCode);
        if (agent == null) {
            sendResult.setMsg("未找到短信服务商配置");
            return sendResult;
        }

        String agentName = agent.getAgentName();
        // 验证码
        String verifyCode = RandomUtil.randomNumbers(4);

        boolean isDev = AppEnv.DEV.equals(pushEnv);
        String verifyPhone = phone;
        if (phone.startsWith(areaCode)) {
            verifyPhone = phone.substring(areaCode.length());
        }
        if (securityService.isMarketAuditorPhone(verifyPhone, areaCode)) {
            // 市场审核人员账号不发验证码 直接返回
            isDev = true;
            isDevSend = false;
        }
        // 判断是否需要真实发送，只有正式环境 或者 测试环境并传真实发送参数才发送
        if (!isDev || isDevSend) {
            if (agentName.equals("aliyun")) {

                String signName = signDO.getSignName();
                if (!"86".equals(areaCode)) {
                    signName = signDO.getInternalSignName();
                }

                log.info("templateCode:{},phone:{},verifyCode:{},signName:{}", templateCode, phone, verifyCode, signName);

                String result = aliyunSmsProvider.sendVoiceSms(templateCode, phone, showNumber, new VoiceSmsCode("芝麻乐", verifyCode), agent.getAccessKey(), agent.getAccessSecret());

                log.info("result:{}", result);
                if (StringUtils.isBlank(result)) {
                    sendResult.setMsg("短信服务商异常");
                    return sendResult;
                }

                // 从json结果读msg
                String message = JSON.parseObject(result).getString("Message");
                if (!StringConstant.OK.equalsIgnoreCase(message)) {
                    sendResult.setMsg(message);
                    return sendResult;
                }
            } else {
                sendResult.setMsg("未开通该服务商的短信功能");
                return sendResult;
            }
        } else {
            // 测试环境模拟延迟
            try {
                Thread.sleep(RandomUtil.randomLong(600, 3000));
            } catch (InterruptedException e) {
                log.error("测试环境 sleep 中断", e);
            }
        }

        // 写入短信记录
        writeHis(action, phone, verifyCode, smsConfig.getSmsAgentId(), channelId, ip, deviceId);

        // 发送成功
        sendResult.setSuccess(true);
        // 测试环境或者审核人员账号需要回填验证码
        if (isDev) {
            sendResult.setData(verifyCode);
        }

        return sendResult;
    }

    /**
     * 手机号验证码校验
     *
     * @param action 短信动作
     * @param phone  手机号
     * @param code   短信验证码
     * @return 是否通过
     */
    @Override
    public Boolean checkSmsCode(SmsActionEnum action, String phone, String code) {
        String codeInRedis = redisService.get(getRedisKey(action, phone));
        Boolean validRes = code.equals(codeInRedis);

        if (!validRes) {
            // 增加错误次数
            redisService.increaseAndExpire(RedisCacheKeys.getSmsInvalidTimes(phone), 1, TimeConstants.MIN_MILLS, TimeUnit.MILLISECONDS);
        }

        return validRes;
    }

    /**
     * 删除短信验证码
     *
     * @param action 短信动作
     * @param phone  手机号
     * @return 是否成功
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean delSmsCode(SmsActionEnum action, String phone) {
        String redisKey = getRedisKey(action, phone);
        return redisService.delKey(redisKey);
    }

    @Override
    public boolean machineValidate(Long channelId, String sessionId, String sig, String token, String scene, String remoteIp) {
        log.info("人机校验入参,channelId:{},sessionId:{},sig:{},token:{},scene:{},remoteIp:{}", channelId, sessionId, sig, token, scene, remoteIp);

        // 获取渠道短信配置
        ChannelSmsConfig smsConfig = channelService.getSmsConfig(channelId);
        if (smsConfig == null || smsConfig.getSmsAgentId() == null) {
            log.info("无短信配置");
            return false;
        }

        // 获取短信配置
        SmsAgentKeySecretName agent = smsConfigService.getAgentKeyAndSecret(smsConfig.getSmsAgentId());
        if (agent == null) {
            log.info("未配置短信");
            return false;
        }
        return aliyunSmsProvider.machineValidate(agent.getAccessKey(), agent.getAccessSecret(), sessionId, sig, token, scene, remoteIp);
    }

    /**
     * 获取手机验证码校验错误次数
     *
     * @param phone
     * @return
     */
    @Override
    public int getPhoneInvalidTimes(String phone) {
        return Optional.ofNullable(redisService.get(RedisCacheKeys.getSmsInvalidTimes(phone))).map(Convert::toInt).orElse(0);
    }

    /**
     * 获取redisKey
     *
     * @param action 类型
     * @param phone  手机号
     */
    private String getRedisKey(SmsActionEnum action, String phone) {
        String redisKey = "";
        switch (action) {
            case LOGIN:
                redisKey = RedisCacheKeys.getSmsLoginKey(phone);
                break;
            case REGISTER:
                redisKey = RedisCacheKeys.getSmsRegKey(phone);
                break;
            case FORGET_PASSWORD:
                redisKey = RedisCacheKeys.getSmsForget(phone);
                break;
            case BIND:
                redisKey = RedisCacheKeys.getSmsBind(phone);
                break;
            default:
                break;
        }
        return redisKey;
    }

    /**
     * 判断是否json
     *
     * @param str 字符串
     */
    public static boolean isJson(String str) {
        try {
            JSONObject.parseObject(str);
            return true;
        } catch (Exception e) {
            return false;
        }
    }

    /**
     * 校验
     *
     * @param action 类型
     * @param phone  手机号
     */
    private boolean check(SmsActionEnum action, String phone) {
        //剩余生存时间(秒)
        long ttl = redisService.getTtl(getRedisKey(action, phone));
        //剩余生存时间(毫秒)
        long remain = ttl * 1000;
        //1分钟
        long oneMin = 1 * 60 * 1000;
        //是否小于
        return remain + oneMin < smsExpireMill;
    }

    /**
     * 写入缓存记录
     *
     * @param action     短信动作
     * @param phone      手机号
     * @param verifyCode 验证码
     * @param agentId    代理商id
     * @param channelId  渠道id
     * @param ip
     * @param deviceId
     * @return 是否成功
     */
    private boolean writeHis(SmsActionEnum action, String phone, String verifyCode, Long agentId, Long channelId, String ip, String deviceId) {
        redisService.set(getRedisKey(action, phone), verifyCode, smsExpireMill, TimeUnit.MILLISECONDS);

        // 写入数据库
        Long curTime = System.currentTimeMillis();
        SmsCodeHisDO codeHisDO = new SmsCodeHisDO();
        codeHisDO.setHisId(identityService.getId());
        codeHisDO.setCodeType(action.getCode());
        codeHisDO.setPhone(phone);
        codeHisDO.setSmsCode(verifyCode);
        codeHisDO.setAgentId(agentId);
        codeHisDO.setChannelId(channelId);
        codeHisDO.setResult(StringConstant.SEND_SUCCESS);
        codeHisDO.setStatus(CommonStatus.Normal);
        codeHisDO.setCreateTime(curTime);
        codeHisDO.setIsDeleted(CommonStatus.Disabled);
        codeHisDO.setSendIp(ip);
        codeHisDO.setDeviceId(deviceId);
        smsCodeHisDAO.insert(codeHisDO);

        // 异步统计
        if (StringUtils.isNotBlank(ip) && StringUtils.isNotBlank(deviceId)) {
            statPlatformService.addSmsSendStat(ip, deviceId);
        }
        return true;
    }

}
