package com.starbuds.server.service.message.impl.biz;

import com.starbuds.server.common.constant.activity.LotteryGamePlayTypeEnum;
import com.starbuds.server.common.constant.message.im.ContentType;
import com.starbuds.server.common.constant.message.im.ImActionType;
import com.starbuds.server.common.pojo.message.im.activity.*;
import com.starbuds.server.common.pojo.message.im.biz.SystemMsg;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.message.biz.LotteryGameImMsgService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.common.utils.CollectionUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;

import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import static com.google.common.base.Preconditions.checkArgument;
import static com.google.common.base.Preconditions.checkNotNull;

/**
 * 抽奖活动IM消息
 *
 * @author aoe
 * @date 2021/11/22
 */
@Slf4j
@Service(version = "1.0.0")
public class LotteryGameImMsgServiceImpl implements LotteryGameImMsgService {

    @Reference(version = "1.0.0")
    private ImService imService;

    @Override
    public boolean sendCountdownImMsg(LotteryGameCountdownImMsg imMsg) {
        return imService.sendAllRoomMsg(imMsg);
    }

    @Override
    public boolean sendComingSoonImMsg(LotteryGameComingSoonImMsg imMsg) {
        return imService.sendAllRoomMsg(imMsg);
    }

    @Override
    public boolean sendLotteryResultImMsg(Long snapId, LotteryGamePlayTypeEnum playType) {
        checkNotNull(snapId, "snapId 不能为 null");
        checkNotNull(playType, "playType 不能为 null");

        LotteryGameLotteryResultImMsg imMsg = new LotteryGameLotteryResultImMsg();
        imMsg.setSnapId(snapId);
        imMsg.setPlayType(playType.value);
        return imService.sendAllRoomMsg(imMsg);
    }

    @Override
    public boolean sendWinningRecordImMsg(WinningRecordParam param) {
        checkNotNull(param);
        checkNotNull(param.getUserId());
        checkNotNull(param.getSource());
        checkArgument(CollectionUtils.isNotEmpty(param.getWinItems()), "中奖物品不能为空");

        SystemMsg systemMsg = getSystemMsg(param);
        return imService.sendCustomImMsg(null, param.getUserId(), systemMsg, null);
    }

    private SystemMsg getSystemMsg(WinningRecordParam param) {
        SystemMsg systemMsg = new SystemMsg();
        systemMsg.setContentType(ContentType.TEXT);
        systemMsg.setTitle(param.getSource().getName());
        systemMsg.setBody(getWinningRecordMsgBody(param.getWinItems()));
        systemMsg.setActionType(ImActionType.NO_ACTION.getValue());
        return systemMsg;
    }

    /**
     * 中奖消息内容
     *
     * @param winItems
     * @return
     */
    private String getWinningRecordMsgBody(List<WinItem> winItems) {
        List<String> items = winItems.stream()
                .filter(Objects::nonNull)
                .filter(item -> item.getQuantity() != null)
                .filter(item -> StringUtils.isNotBlank(item.getPrizeName()))
                .map(item -> String.format("【%s】%s个", item.getPrizeName(), item.getQuantity()))
                .collect(Collectors.toList());
        return StringUtils.join(items, ",");
    }

}
