package com.starbuds.server.service.message.netease;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.http.Header;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;

/**
 * 备注
 *
 * @author Lay
 * @date 2019/12/10
 */
@Slf4j
public abstract class AbstractNeteaseClient implements NeteaseClient {
    private String appKey;
    private String appSecret;

    public AbstractNeteaseClient(String appKey, String appSecret) {
        this.appKey = appKey;
        this.appSecret = appSecret;
    }

    /**
     * 执行请求
     *
     * @param request
     * @return
     */
    @Override
    public <T extends NeteaseResponse> T execute(NeteaseRequest<T> request) {

        // 随机数
        String nonce = RandomUtil.randomNumbers(32);

        // 时间戳
        String curTime = String.valueOf(Convert.toInt(System.currentTimeMillis() / 1000L));

        // 构造基本请求头
        HttpResponse response = HttpRequest.post(request.getApiUrl())
                .header(Header.CONTENT_TYPE, "application/x-www-form-urlencoded;charset=utf-8;")
                .header("AppKey", appKey)
                .header("Nonce", nonce)
                .header("CurTime", curTime)
                .header("CheckSum", SecureUtil.sha1(String.format("%s%s%s", appSecret, nonce, curTime)).toLowerCase())
                .form(request.getParamsMap())
                .execute();

        if (response.isOk()) {
            return JSON.parseObject(response.body(), request.getResponseClass());
        }
        return null;
    }

    private static final char[] HEX_DIGITS = {'0', '1', '2', '3', '4', '5',
            '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};

    private static String getFormattedText(byte[] bytes) {
        int len = bytes.length;
        StringBuilder buf = new StringBuilder(len * 2);
        for (int j = 0; j < len; j++) {
            buf.append(HEX_DIGITS[(bytes[j] >> 4) & 0x0f]);
            buf.append(HEX_DIGITS[bytes[j] & 0x0f]);
        }
        return buf.toString();
    }
}
