package com.starbuds.server.service.message.netease;

import lombok.Getter;
import lombok.Setter;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

/**
 * 请求响应
 *
 * @author Lay
 * @date 2019/12/9
 */
public abstract class NeteaseResponse implements Serializable {

    /**
     * 错误码
     */
    @Getter
    @Setter
    private Integer code;

    /**
     * 错误码描述
     *
     * @return
     */
    public String getCodeDesc() {
        if (code != null && CODE_DESC.containsKey(code)) {
            return CODE_DESC.get(code);
        }
        return "未知错误";
    }

    public boolean isSuccess() {
        return code != null && code.equals(200);
    }

    private static final Map<Integer, String> CODE_DESC = new HashMap<Integer, String>() {
        {
            put(200, "操作成功");
            put(201, "客户端版本不对，需升级sdk");
            put(301, "被封禁");
            put(302, "用户名或密码错误");
            put(315, "IP限制");
            put(403, "非法操作或没有权限");
            put(404, "对象不存在");
            put(405, "参数长度过长");
            put(406, "对象只读");
            put(408, "客户端请求超时");
            put(413, "验证失败(短信服务)");
            put(414, "参数错误");
            put(415, "客户端网络问题");
            put(416, "频率控制");
            put(417, "重复操作");
            put(418, "通道不可用(短信服务)");
            put(419, "数量超过上限");
            put(422, "账号被禁用");
            put(423, "帐号被禁言");
            put(431, "HTTP重复请求");
            put(500, "服务器内部错误");
            put(503, "服务器繁忙");
            put(508, "消息撤回时间超限");
            put(509, "无效协议");
            put(514, "服务不可用");
            put(998, "解包错误");
            put(999, "打包错误");
            put(801, "群人数达到上限");
            put(802, "没有权限");
            put(803, "群不存在");
            put(804, "用户不在群");
            put(805, "群类型不匹配");
            put(806, "创建群数量达到限制");
            put(807, "群成员状态错误");
            put(808, "申请成功");
            put(809, "已经在群内");
            put(810, "邀请成功");
            put(811, "@账号数量超过限制");
            put(812, "群禁言，普通成员不能发送消息");
            put(813, "群拉人部分成功");
            put(814, "禁止使用群组已读服务");
            put(815, "群管理员人数超过上限");
            put(9102, "通道失效");
            put(9103, "已经在他端对这个呼叫响应过了");
            put(11001, "通话不可达，对方离线状态");
            put(13001, "IM主连接状态异常");
            put(13002, "聊天室状态异常");
            put(13003, "账号在黑名单中,不允许进入聊天室");
            put(13004, "在禁言列表中,不允许发言");
            put(13005, "用户的聊天室昵称、头像或成员扩展字段被反垃圾");
            put(10431, "输入email不是邮箱");
            put(10432, "输入mobile不是手机号码");
            put(10433, "注册输入的两次密码不相同");
            put(10434, "企业不存在");
            put(10435, "登陆密码或帐号不对");
            put(10436, "app不存在");
            put(10437, "email已注册");
            put(10438, "手机号已注册");
            put(10441, "app名字已经存在");
        }
    };
}
