package com.starbuds.server.service.message.netease.request.chatroom;

import com.alibaba.fastjson.annotation.JSONField;
import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.response.chatroom.NeteaseChatroomAddRobotResponse;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/12 17:38
 * @Description 往聊天室内添加机器人
 */
@Data
public class NeteaseChatroomAddRobotRequest implements NeteaseRequest<NeteaseChatroomAddRobotResponse> {

    /**
     * 聊天室id
     *
     * @required
     */
    @JSONField(name = "roomid")
    private long roomId;

    /**
     * 机器人账号accid列表，必须是有效账号，账号数量上限100个 JSONArray
     *
     * @required
     */
    @JSONField(name = "accids")
    private String accIds;

    /**
     * 机器人信息扩展字段，请使用json格式，长度4096字符
     */
    private String roleExt;

    /**
     * 机器人进入聊天室通知的扩展字段，请使用json格式，长度2048字符
     */
    private String notifyExt;


    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.CHATROOM_ADD_ROBOT;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("roomid", roomId);
            put("accids", accIds);
            put("roleExt", roleExt);
            put("notifyExt", notifyExt);
        }};
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseChatroomAddRobotResponse> getResponseClass() {
        return NeteaseChatroomAddRobotResponse.class;
    }
}
