package com.starbuds.server.service.message.netease.request.chatroom;

import com.alibaba.fastjson.annotation.JSONField;
import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.response.chatroom.NeteaseChatroomAddrResponse;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/12 17:09
 * @Description 请求聊天室地址
 */
@Data
public class NeteaseChatroomAddrRequest implements NeteaseRequest<NeteaseChatroomAddrResponse> {

    /**
     * 聊天室id
     *
     * @required
     */
    @JSONField(name = "roomid")
    private long roomId;

    /**
     * 进入聊天室的账号
     *
     * @required
     */
    @JSONField(name = "accid")
    private String accId;

    /**
     * 1:weblink（客户端为web端时使用）; 2:commonlink（客户端为非web端时使用）;3:wechatlink(微信小程序使用), 默认1
     *
     * @required
     */
    @JSONField(name = "clienttype")
    private int clientType;

    /**
     * 客户端ip，传此参数时，会根据用户ip所在地区，返回合适的地址
     */
    @JSONField(name = "clientip")
    private String clientIp;

    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.CHATROOM_REQUEST_ADDR;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("roomid", roomId);
            put("accid", accId);
            put("clienttype", clientType);
            put("clientip", clientIp);
        }};
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseChatroomAddrResponse> getResponseClass() {
        return NeteaseChatroomAddrResponse.class;
    }
}
