package com.starbuds.server.service.message.netease.request.chatroom;

import com.alibaba.fastjson.annotation.JSONField;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.response.chatroom.NeteaseChatroomCreateResponse;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * 备注
 *
 * @author Lay
 * @date 2019/12/10
 */
@Data
public class NeteaseChatroomCreateRequest implements NeteaseRequest<NeteaseChatroomCreateResponse> {

    /**
     * 聊天室属主的账号 accid
     *
     * @required
     */
    private String creator;

    /**
     * 聊天室名称，长度限制128个字符
     *
     * @required
     */
    private String name;

    /**
     * 公告，长度限制4096个字符
     */
    private String announcement;

    /**
     * 直播地址，长度限制1024个字符
     */
    @JSONField(name = "broadcasturl")
    private String broadCastUrl;

    /**
     * 扩展字段，最长4096字符
     */
    private String ext;

    /**
     * 队列管理权限：0:所有人都有权限变更队列，1:只有主播管理员才能操作变更。默认0
     */
    @JSONField(name = "queuelevel")
    private int queueLevel;

    /**
     * 获取接口请求地址
     *
     * @return
     */
    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.CHATROOM_CREATE;
    }

    /**
     * 获取请求参数
     *
     * @return
     */
    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>(6) {
            {
                put("creator", creator);
                put("name", name);
                put("announcement", announcement);
                put("broadcasturl", broadCastUrl);
                put("ext", ext);
                put("queuelevel", queueLevel);
            }
        };
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseChatroomCreateResponse> getResponseClass() {
        return NeteaseChatroomCreateResponse.class;
    }
}
