package com.starbuds.server.service.message.netease.request.chatroom;

import com.alibaba.fastjson.annotation.JSONField;
import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.response.chatroom.NeteaseChatroomInfoBatchResponse;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/12 16:13
 * @Description 批量查询聊天室信息
 */
@Data
public class NeteaseChatroomInfoBatchRequest implements NeteaseRequest<NeteaseChatroomInfoBatchResponse> {

    /**
     * 多个roomid，格式为：["6001","6002","6003"]（JSONArray对应的roomid，如果解析出错，会报414错误），限20个roomid
     *
     * @required
     */
    @JSONField(name = "roomids")
    private String roomIds;

    /**
     * 是否需要返回在线人数，true或false，默认false
     */
    private String needOnlineUserCount;

    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.CHATROOM_INFO_BATCH;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("roomids", roomIds);
            put("needOnlineUserCount", needOnlineUserCount);
        }};
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseChatroomInfoBatchResponse> getResponseClass() {
        return NeteaseChatroomInfoBatchResponse.class;
    }
}
