package com.starbuds.server.service.message.netease.request.chatroom;

import com.alibaba.fastjson.annotation.JSONField;
import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.response.chatroom.NeteaseChatroomMemberPageResponse;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/13 14:07
 * @Description 分页获取成员列表
 */
@Data
public class NeteaseChatroomMemberPageRequest implements NeteaseRequest<NeteaseChatroomMemberPageResponse> {

    /**
     * 聊天室id
     *
     * @required
     */
    @JSONField(name = "roomid")
    private long roomId;

    /**
     * 需要查询的成员类型,0:固定成员;1:非固定成员;2:仅返回在线的固定成员
     *
     * @required
     */
    private int type;

    /**
     * 单位毫秒，按时间倒序最后一个成员的时间戳,0表示系统当前时间
     *
     * @required
     */
    @JSONField(name = "endtime")
    private long endTime;

    /**
     * 返回条数，<=100
     *
     * @required
     */
    private long limit;

    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.CHATROOM_MEMBERS_BY_PAGE;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("roomid", roomId);
            put("type", type);
            put("endtime", endTime);
            put("limit", limit);
        }};
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseChatroomMemberPageResponse> getResponseClass() {
        return NeteaseChatroomMemberPageResponse.class;
    }
}
