package com.starbuds.server.service.message.netease.request.chatroom;

import com.alibaba.fastjson.annotation.JSONField;
import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.response.chatroom.NeteaseChatroomQueueBatchUpdateElementResponse;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/13 14:26
 * @Description 批量更新聊天室队列元素
 */
@Data
public class NeteaseChatroomQueueBatchUpdateElementRequest implements NeteaseRequest<NeteaseChatroomQueueBatchUpdateElementResponse> {

    /**
     * 聊天室id
     *
     * @required
     */
    @JSONField(name = "roomid")
    private long roomId;

    /**
     * 操作者accid,必须是管理员或创建者
     *
     * @required
     */
    private String operator;

    /**
     * 更新的key-value对，最大200个，示例：{"k1":"v1","k2":"v2"}
     *
     * @required
     */
    private String elements;

    /**
     * true或false,是否需要发送更新通知事件，默认true
     */
    private Boolean needNotify;

    /**
     * 通知事件扩展字段，长度限制2048
     */
    private String notifyExt;

    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.CHATROOM_QUEUE_BATCH_UPDATE_ELEMENTS;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("roomid", roomId);
            put("operator", operator);
            put("elements", elements);
            put("needNotify", needNotify);
            put("notifyExt", notifyExt);
        }};
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseChatroomQueueBatchUpdateElementResponse> getResponseClass() {
        return NeteaseChatroomQueueBatchUpdateElementResponse.class;
    }
}
