package com.starbuds.server.service.message.netease.request.chatroom;

import com.alibaba.fastjson.annotation.JSONField;
import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.response.chatroom.NeteaseChatroomQueueOfferResponse;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/13 11:17
 * @Description 往聊天室有序队列中新加或更新元素
 */
@Data
public class NeteaseChatroomQueueOfferRequest implements NeteaseRequest<NeteaseChatroomQueueOfferResponse> {

    /**
     * 聊天室id
     *
     * @required
     */
    @JSONField(name = "roomid")
    private long roomId;

    /**
     * elementKey,新元素的UniqKey,长度限制128字符
     *
     * @required
     */
    private String key;

    /**
     * elementValue,新元素内容，长度限制4096字符
     *
     * @required
     */
    private String value;

    /**
     * 提交这个新元素的操作者accid，默认为该聊天室的创建者，若operator对应的帐号不存在，会返回404错误。
     * 若指定的operator不在线，则添加元素成功后的通知事件中的操作者默认为聊天室的创建者；若指定的operator在线，则通知事件的操作者为operator。
     */
    private String operator;

    /**
     * 这个新元素的提交者operator的所有聊天室连接在从该聊天室掉线或者离开该聊天室的时候，提交的元素是否需要删除。
     * true：需要删除；false：不需要删除。默认false。
     * 当指定该参数为true时，若operator当前不在该聊天室内，则会返回403错误。
     */
    @JSONField(name = "transient")
    private String transients;

    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.CHATROOM_QUEUE_OFFER;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("roomid", roomId);
            put("key", key);
            put("value", value);
            put("operator", operator);
            put("transient", transients);
        }};
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseChatroomQueueOfferResponse> getResponseClass() {
        return NeteaseChatroomQueueOfferResponse.class;
    }
}
