package com.starbuds.server.service.message.netease.request.chatroom;

import com.alibaba.fastjson.annotation.JSONField;
import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.response.chatroom.NeteaseChatroomSetMemberRoleResponse;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/12 17:00
 * @Description 设置聊天室内用户角色
 */
@Data
public class NeteaseChatroomSetMemberRoleRequest implements NeteaseRequest<NeteaseChatroomSetMemberRoleResponse> {

    /**
     * 聊天室id
     *
     * @required
     */
    @JSONField(name = "roomid")
    private long roomId;

    /**
     * 操作者账号accid
     *
     * @required
     */
    private String operator;

    /**
     * 被操作者账号accid
     *
     * @required
     */
    private String target;

    /**
     * 操作：
     * 1: 设置为管理员，operator必须是创建者
     * 2:设置普通等级用户，operator必须是创建者或管理员
     * -1:设为黑名单用户，operator必须是创建者或管理员
     * -2:设为禁言用户，operator必须是创建者或管理员
     *
     * @required
     */
    private int opt;

    /**
     * true或false，true:设置；false:取消设置；
     * 执行“取消”设置后，若成员非禁言且非黑名单，则变成游客
     *
     * @required
     */
    @JSONField(name = "optvalue")
    private String optValue;

    /**
     * 通知扩展字段，长度限制2048，请使用json格式
     */
    private String notifyExt;


    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.CHATROOM_SET_MEMBER_ROLE;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("roomid", roomId);
            put("operator", operator);
            put("target", target);
            put("opt", opt);
            put("optvalue", optValue);
            put("notifyExt", notifyExt);
        }};
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseChatroomSetMemberRoleResponse> getResponseClass() {
        return NeteaseChatroomSetMemberRoleResponse.class;
    }
}
