package com.starbuds.server.service.message.netease.request.chatroom;

import com.alibaba.fastjson.annotation.JSONField;
import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.response.chatroom.NeteaseChatroomTemporaryMuteResponse;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/13 11:08
 * @Description 设置临时禁言状态
 */
@Data
public class NeteaseChatroomTemporaryMuteRequest implements NeteaseRequest<NeteaseChatroomTemporaryMuteResponse> {

    /**
     * 聊天室id
     *
     * @required
     */
    @JSONField(name = "roomid")
    private long roomId;

    /**
     * 操作者accid,必须是管理员或创建者
     *
     * @required
     */
    private String operator;

    /**
     * 被禁言的目标账号accid
     *
     * @required
     */
    private String target;

    /**
     * 0:解除禁言;>0设置禁言的秒数，不能超过2592000秒(30天)
     *
     * @required
     */
    private long muteDuration;

    /**
     * 操作完成后是否需要发广播，true或false，默认true
     */
    private String needNotify;

    /**
     * 通知广播事件中的扩展字段，长度限制2048字符
     */
    private String notifyExt;

    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.CHATROOM_TEMPORARY_MUTE;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("roomid", roomId);
            put("operator", operator);
            put("target", target);
            put("muteDuration", muteDuration);
            put("needNotify", needNotify);
            put("notifyExt", notifyExt);
        }};
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseChatroomTemporaryMuteResponse> getResponseClass() {
        return NeteaseChatroomTemporaryMuteResponse.class;
    }
}
