package com.starbuds.server.service.message.netease.request.chatroom;

import com.alibaba.fastjson.annotation.JSONField;
import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.response.chatroom.NeteaseChatroomTopnResponse;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/13 13:40
 * @Description 查询聊天室统计指标TopN
 */
@Data
public class NeteaseChatroomTopnRequest implements NeteaseRequest<NeteaseChatroomTopnResponse> {

    /**
     * topn值，可选值 1~500，默认值100
     */
    @JSONField(name = "topn")
    private int topN;

    /**
     * 需要查询的指标所在的时间坐标点，不提供则默认当前时间，单位秒/毫秒皆可
     */
    private long timestamp;

    /**
     * 统计周期，可选值包括 hour/day, 默认hour
     */
    private String period;

    /**
     * 取排序值,可选值 active/enter/message,分别表示按日活排序，进入人次排序和消息数排序， 默认active
     */
    @JSONField(name = "orderby")
    private String orderBy;

    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.CHATROOM_TOPN;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("topn", topN);
            put("timestamp", timestamp);
            put("period", period);
            put("orderby", orderBy);
        }};
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseChatroomTopnResponse> getResponseClass() {
        return NeteaseChatroomTopnResponse.class;
    }
}
