package com.starbuds.server.service.message.netease.request.chatroom;

import com.alibaba.fastjson.annotation.JSONField;
import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.response.chatroom.NeteaseChatroomUpdateResponse;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/12 16:45
 * @Description 更新聊天室信息
 */
@Data
public class NeteaseChatroomUpdateRequest implements NeteaseRequest<NeteaseChatroomUpdateResponse> {

    /**
     * 聊天室id
     *
     * @required
     */
    @JSONField(name = "roomid")
    private long roomId;

    /**
     * 聊天室名称，长度限制128个字符
     */
    private String name;

    /**
     * 公告，长度限制4096个字符
     */
    private String announcement;

    /**
     * 直播地址，长度限制1024个字符
     */
    @JSONField(name = "broadcasturl")
    private String broadCastUrl;

    /**
     * 扩展字段，长度限制4096个字符
     */
    private String ext;

    /**
     * true或false,是否需要发送更新通知事件，默认true
     */
    private String needNotify;

    /**
     * 通知事件扩展字段，长度限制2048
     */
    private String notifyExt;

    /**
     * 队列管理权限：0:所有人都有权限变更队列，1:只有主播管理员才能操作变更
     */
    @JSONField(name = "queuelevel")
    private int queueLevel;


    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.CHATROOM_UPDATE;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("roomid", roomId);
            put("name", name);
            put("announcement", announcement);
            put("broadcasturl", broadCastUrl);
            put("ext", ext);
            put("needNotify", needNotify);
            put("notifyExt", notifyExt);
            put("queuelevel", queueLevel);
        }};
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseChatroomUpdateResponse> getResponseClass() {
        return NeteaseChatroomUpdateResponse.class;
    }
}
