package com.starbuds.server.service.message.netease.request.chatroom;

import com.alibaba.fastjson.annotation.JSONField;
import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.response.chatroom.NeteaseChatroomUpdateRoomRoleResponse;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/13 14:21
 * @Description 变更聊天室内的角色信息
 */
@Data
public class NeteaseChatroomUpdateRoomRoleRequest implements NeteaseRequest<NeteaseChatroomUpdateRoomRoleResponse> {

    /**
     * 聊天室id
     *
     * @required
     */
    @JSONField(name = "roomid")
    private Long roomId;

    /**
     * 需要变更角色信息的accid
     *
     * @required
     */
    @JSONField(name = "accid")
    private String accId;

    /**
     * 变更的信息是否需要持久化，默认false，仅对聊天室固定成员生效
     */
    private Boolean save;

    /**
     * 是否需要做通知
     */
    private Boolean needNotify;

    /**
     * 通知的内容，长度限制2048
     */
    private String notifyExt;

    /**
     * 聊天室室内的角色信息：昵称，不超过64个字符
     */
    private String nick;

    /**
     * 聊天室室内的角色信息：头像
     */
    @JSONField(name = "avator")
    private String avatar;

    /**
     * 聊天室室内的角色信息：开发者扩展字段
     */
    private String ext;

    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.CHATROOM_UPDATE_MY_ROOM_ROLE;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("roomid", roomId);
            put("accid", accId);
            put("save", save);
            put("needNotify", needNotify);
            put("notifyExt", notifyExt);
            put("nick", nick);
            put("avator", avatar);
            put("ext", ext);
        }};
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseChatroomUpdateRoomRoleResponse> getResponseClass() {
        return NeteaseChatroomUpdateRoomRoleResponse.class;
    }
}
