package com.starbuds.server.service.message.netease.request.msg;

import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.response.msg.NeteaseMsgBroadcastResponse;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/11 14:12
 * @Description 发送广播消息
 * 1、使用广播消息前，请务必阅读注意事项，详见关于广播消息。
 * 2、广播消息，可以对应用内的所有用户发送广播消息，广播消息目前暂不支持第三方推送（APNS、小米、华为等）；
 * 3、广播消息支持离线存储，并可以自定义设置离线存储的有效期，最多保留最近100条离线广播消息；
 * 4、此接口受频率控制，一个应用一分钟最多调用10次，一天最多调用1000次，超过会返回416状态码；
 * 5、该功能目前需申请开通，详情可咨询您的客户经理。
 */
@Data
public class NeteaseMsgBroadcastRequest implements NeteaseRequest<NeteaseMsgBroadcastResponse> {

    /**
     * 广播消息内容，最大4096字符
     *
     * @required
     */
    private String body;

    /**
     * 发送者accid, 用户帐号，最大长度32字符，必须保证一个APP内唯一
     */
    private String from;

    /**
     * 是否存离线，true或false，默认false
     */
    private String isOffline;

    /**
     * 存离线状态下的有效期，单位小时，默认7天
     */
    private int ttl;

    /**
     * 目标客户端，默认所有客户端，jsonArray，格式：["ios","aos","pc","web","mac"]
     */
    private String targetOs;

    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.BROAD_CAST_MSG;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("body", body);
            put("from", from);
            put("isOffline", isOffline);
            put("ttl", ttl);
            put("targetOs", targetOs);
        }};
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseMsgBroadcastResponse> getResponseClass() {
        return NeteaseMsgBroadcastResponse.class;
    }
}
