package com.starbuds.server.service.message.netease.request.msg;

import com.alibaba.fastjson.annotation.JSONField;
import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.response.msg.NeteaseMsgMultipartUploadResponse;
import lombok.Data;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/11 13:49
 * @Description 文件上传（multipart方式）
 */
@Data
public class NeteaseMsgMultipartUploadRequest implements NeteaseRequest<NeteaseMsgMultipartUploadResponse> {

    /**
     * 最大15M的字符流  Multipart
     *
     * @required
     */
    private File content;

    /**
     * 上传文件类型
     */
    private String type;

    /**
     * 返回的url是否需要为https的url，true或false，默认false
     */
    @JSONField(name = "ishttps")
    private String isHttps;

    /**
     * 文件过期时长，单位：秒，必须大于等于86400
     */
    private Integer expireSec;

    /**
     * 文件的应用场景，不超过32个字符
     */
    private String tag;

    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.MSG_MULTIPART_UPLOAD;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("content", content);
            put("type", type);
            put("ishttps", isHttps);
            put("expireSec", expireSec);
            put("tag", tag);
        }};
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseMsgMultipartUploadResponse> getResponseClass() {
        return NeteaseMsgMultipartUploadResponse.class;
    }
}
