package com.starbuds.server.service.message.netease.request.msg;

import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.response.msg.NeteaseMsgNosJobDelResponse;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/11 13:57
 * @Description 上传NOS文件清理任务
 * 上传NOS文件清理任务，按时间范围和文件类下、场景清理符合条件的文件
 * 每天提交的任务数量有限制，请合理规划
 */
@Data
public class NeteaseMsgNosJobDelRequest implements NeteaseRequest<NeteaseMsgNosJobDelResponse> {

    /**
     * 被清理文件的开始时间，必须小于endTime且大于0，endTime和startTime不能之间不能超过7天
     *
     * @required
     */
    private Long startTime;

    /**
     * 被清理文件的结束时间，必须大于startTime且早于今天（即只可以清理今天以前的文件）
     *
     * @required
     */
    private Long endTime;

    /**
     * 被清理的文件类型，文件类型包含contentType则被清理 如原始文件类型为"image/png"，contentType参数为"image",则满足被清理条件
     */
    private String contentType;

    /**
     * 被清理文件的应用场景，完全相同才被清理 如上传文件时知道场景为"usericon",tag参数为"usericon"，则满足被清理条件
     */
    private String tag;


    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.JOB_NOS_DEL;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("startTime", startTime);
            put("endTime", endTime);
            put("contentType", contentType);
            put("tag", tag);
        }};
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseMsgNosJobDelResponse> getResponseClass() {
        return NeteaseMsgNosJobDelResponse.class;
    }
}
