package com.starbuds.server.service.message.netease.request.msg;

import com.alibaba.fastjson.annotation.JSONField;
import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.response.msg.NeteaseMsgRecallResponse;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/11 14:05
 * @Description 消息撤回
 */
@Data
public class NeteaseMsgRecallRequest implements NeteaseRequest<NeteaseMsgRecallResponse> {


    /**
     * 要撤回消息的msgid
     *
     * @required
     */
    @JSONField(name = "deleteMsgid")
    private String deleteMsgId;

    /**
     * 要撤回消息的创建时间
     *
     * @required
     */
    @JSONField(name = "timetag")
    private long timeTag;

    /**
     * 7:表示点对点消息撤回，8:表示群消息撤回，其它为参数错误
     *
     * @required
     */

    private int type;

    /**
     * 发消息的accid
     *
     * @required
     */
    private String from;

    /**
     * 如果点对点消息，为接收消息的accid,如果群消息，为对应群的tid
     *
     * @required
     */
    private String to;

    /**
     * 可以带上对应的描述
     */
    private String msg;

    /**
     * 1表示绕过撤回时间检测，其它为非法参数，最多撤回近30天内的消息。如果需要撤回时间检测，不填即可。
     */
    private String ignoreTime;

    /**
     * 推送文案，android以此为推送显示文案；ios若未填写payload，显示文案以pushcontent为准。超过500字符后，会对文本进行截断。
     */
    @JSONField(name = "pushcontent")
    private String pushContent;

    /**
     * 推送对应的payload,必须是JSON,不超过2K字符
     */
    private String payload;

    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.MSG_RECALL;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("deleteMsgid", deleteMsgId);
            put("timetag", timeTag);
            put("type", type);
            put("from", from);
            put("to", to);
            put("msg", msg);
            put("ignoreTime", ignoreTime);
            put("pushcontent", pushContent);
            put("payload", payload);
        }};
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseMsgRecallResponse> getResponseClass() {
        return NeteaseMsgRecallResponse.class;
    }
}
