package com.starbuds.server.service.message.netease.request.msg;

import com.alibaba.fastjson.annotation.JSONField;
import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.response.msg.NeteaseMsgSendBatchResponse;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/11 11:18
 * @Description 批量发送点对点普通消息
 */
@Data
public class NeteaseMsgSendBatchRequest implements NeteaseRequest<NeteaseMsgSendBatchResponse> {

    /**
     * 发送者accid，用户帐号，最大32字符，
     * 必须保证一个APP内唯一
     *
     * @required
     */
    @JSONField(name = "fromAccid")
    private String fromAccId;

    /**
     * ["aaa","bbb"]（JSONArray对应的accid，如果解析出错，会报414错误），限500人
     *
     * @required
     */
    @JSONField(name = "toAccids")
    private String toAccIds;

    /**
     * 0 表示文本消息,
     * 1 表示图片，
     * 2 表示语音，
     * 3 表示视频，
     * 4 表示地理位置信息，
     * 6 表示文件，
     * 100 自定义消息类型
     *
     * @required
     */
    private int type;

    /**
     * 最大长度5000字符，JSON格式。
     * 具体请参考： 消息格式示例
     * 消息格式示例
     * 1.文本消息(type = 0)
     * {
     * "msg":"哈哈哈"//消息内容
     * }
     * 2. 图片消息(type = 1)
     * {
     * "name":"图片发送于2015-05-07 13:59",   //图片name
     * "md5":"9894907e4ad9de4678091277509361f7",    //图片文件md5
     * "url":"http://nimtest.nos.netease.com/cbc500e8-e19c-4b0f-834b-c32d4dc1075e",    //生成的url
     * "ext":"jpg",    //图片后缀
     * "w":6814,    //宽
     * "h":2332,    //高
     * "size":388245    //图片大小
     * }
     * 3. 语音消息(type = 2)
     * {
     * "dur":4551,        //语音持续时长ms
     * "md5":"87b94a090dec5c58f242b7132a530a01",    //语音文件的md5值
     * "url":"http://nimtest.nos.netease.com/a2583322-413d-4653-9a70-9cabdfc7f5f9",    //生成的url
     * "ext":"aac",        //语音消息格式，只能是aac格式
     * "size":16420        //语音文件大小
     * }
     * 4. 视频消息(type = 3)
     * {
     * "dur":8003,        //视频持续时长ms
     * "md5":"da2cef3e5663ee9c3547ef5d127f7e3e",    //视频文件的md5值
     * "url":"http://nimtest.nos.netease.com/21f34447-e9ac-4871-91ad-d9f03af20412",    //生成的url
     * "w":360,    //宽
     * "h":480,    //高
     * "ext":"mp4",    //视频格式
     * "size":16420    //视频文件大小
     * }
     * 5. 发送地理位置消息(type = 4)
     * {
     * "title":"中国 浙江省 杭州市 网商路 599号",    //地理位置title
     * "lng":120.1908686708565,        // 经度
     * "lat":30.18704515647036            // 纬度
     * }
     * 6. 发送文件消息(type = 6)
     * {
     * "name":"BlizzardReg.ttf",    //文件名
     * "md5":"79d62a35fa3d34c367b20c66afc2a500", //文件MD5
     * "url":"http://nimtest.nos.netease.com/08c9859d-183f-4daa-9904-d6cacb51c95b", //文件URL
     * "ext":"ttf",    //文件后缀类型
     * "size":91680    //大小
     * }
     * 7. 发送第三方自定义消息(type = 100)
     * //第三方定义的body体，json格式
     * {
     * "myKey":myValue
     * }
     *
     * @required
     */
    private String body;

    /**
     * 发消息时特殊指定的行为选项,Json格式，可用于指定消息的漫游，存云端历史，发送方多端同步，推送，消息抄送等特殊行为;option中字段不填时表示默认值 option示例:
     * <p>
     * {"push":false,"roam":true,"history":false,"sendersync":true,"route":false,"badge":false,"needPushNick":true}
     * <p>
     * 字段说明：
     * 1. roam: 该消息是否需要漫游，默认true（需要app开通漫游消息功能）；
     * 2. history: 该消息是否存云端历史，默认true；
     * 3. sendersync: 该消息是否需要发送方多端同步，默认true；
     * 4. push: 该消息是否需要APNS推送或安卓系统通知栏推送，默认true；
     * 5. route: 该消息是否需要抄送第三方；默认true (需要app开通消息抄送功能);
     * 6. badge:该消息是否需要计入到未读计数中，默认true;
     * 7. needPushNick: 推送文案是否需要带上昵称，不设置该参数时默认true;
     * 8. persistent: 是否需要存离线消息，不设置该参数时默认true。
     */
    private String option;

    /**
     * 推送文案，最长500个字符，android以此为推送显示文案；ios若未填写payload，显示文案以pushcontent为准。
     */
    @JSONField(name = "pushcontent")
    private String pushContent;

    /**
     * ios 推送对应的payload,必须是JSON,不能超过2k字符
     */
    private String payload;

    /**
     * 开发者扩展字段，长度限制1024字符
     */
    private String ext;

    /**
     * 可选，反垃圾业务ID，实现“单条消息配置对应反垃圾”，若不填则使用原来的反垃圾配置
     */
    private String bid;

    /**
     * 可选，单条消息是否使用易盾反垃圾，可选值为0。
     * 0：（在开通易盾的情况下）不使用易盾反垃圾而是使用通用反垃圾，包括自定义消息。
     * <p>
     * 若不填此字段，即在默认情况下，若应用开通了易盾反垃圾功能，则使用易盾反垃圾来进行垃圾消息的判断
     */
    private int useYidun;

    /**
     * 是否需要返回消息ID
     * false：不返回消息ID（默认值）
     * true：返回消息ID（toAccids包含的账号数量不可以超过100个）
     */
    @JSONField(name = "returnMsgid")
    private Boolean returnMsgId;


    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.SEND_BATCH_MSG;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("fromAccid", fromAccId);
            put("toAccids", toAccIds);
            put("type", type);
            put("body", body);
            put("option", option);
            put("pushcontent", pushContent);
            put("payload", payload);
            put("ext", ext);
            put("bid", bid);
            put("useYidun", useYidun);
            put("returnMsgid", returnMsgId);
        }};

    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseMsgSendBatchResponse> getResponseClass() {
        return NeteaseMsgSendBatchResponse.class;
    }
}
