package com.starbuds.server.service.message.netease.request.msg;

import com.alibaba.fastjson.annotation.JSONField;
import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.response.msg.NeteaseMsgSendResponse;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/11 10:40
 * @Description 发送普通消息
 */
@Data
public class NeteaseMsgSendRequest implements NeteaseRequest<NeteaseMsgSendResponse> {

    /**
     * 发送者accid，用户帐号，最大32字符，
     * 必须保证一个APP内唯一
     *
     * @required
     */
    private String from;

    /**
     * 0：点对点个人消息，1：群消息（高级群），其他返回414
     *
     * @required
     */
    private int ope;

    /**
     * ope==0是表示accid即用户id，ope==1表示tid即群id
     *
     * @required
     */
    private String to;

    /**
     * 0 表示文本消息,
     * 1 表示图片，
     * 2 表示语音，
     * 3 表示视频，
     * 4 表示地理位置信息，
     * 6 表示文件，
     * 100 自定义消息类型（特别注意，对于未对接易盾反垃圾功能的应用，该类型的消息不会提交反垃圾系统检测）
     *
     * @required
     */
    private int type;

    /**
     * 最大长度5000字符，JSON格式。具体请参考： 消息格式示例
     * 消息格式示例
     * 1.文本消息(type = 0)
     * {
     * "msg":"哈哈哈"//消息内容
     * }
     * 2. 图片消息(type = 1)
     * {
     * "name":"图片发送于2015-05-07 13:59",   //图片name
     * "md5":"9894907e4ad9de4678091277509361f7",    //图片文件md5
     * "url":"http://nimtest.nos.netease.com/cbc500e8-e19c-4b0f-834b-c32d4dc1075e",    //生成的url
     * "ext":"jpg",    //图片后缀
     * "w":6814,    //宽
     * "h":2332,    //高
     * "size":388245    //图片大小
     * }
     * 3. 语音消息(type = 2)
     * {
     * "dur":4551,        //语音持续时长ms
     * "md5":"87b94a090dec5c58f242b7132a530a01",    //语音文件的md5值
     * "url":"http://nimtest.nos.netease.com/a2583322-413d-4653-9a70-9cabdfc7f5f9",    //生成的url
     * "ext":"aac",        //语音消息格式，只能是aac格式
     * "size":16420        //语音文件大小
     * }
     * 4. 视频消息(type = 3)
     * {
     * "dur":8003,        //视频持续时长ms
     * "md5":"da2cef3e5663ee9c3547ef5d127f7e3e",    //视频文件的md5值
     * "url":"http://nimtest.nos.netease.com/21f34447-e9ac-4871-91ad-d9f03af20412",    //生成的url
     * "w":360,    //宽
     * "h":480,    //高
     * "ext":"mp4",    //视频格式
     * "size":16420    //视频文件大小
     * }
     * 5. 发送地理位置消息(type = 4)
     * {
     * "title":"中国 浙江省 杭州市 网商路 599号",    //地理位置title
     * "lng":120.1908686708565,        // 经度
     * "lat":30.18704515647036            // 纬度
     * }
     * 6. 发送文件消息(type = 6)
     * {
     * "name":"BlizzardReg.ttf",    //文件名
     * "md5":"79d62a35fa3d34c367b20c66afc2a500", //文件MD5
     * "url":"http://nimtest.nos.netease.com/08c9859d-183f-4daa-9904-d6cacb51c95b", //文件URL
     * "ext":"ttf",    //文件后缀类型
     * "size":91680    //大小
     * }
     * 7. 发送第三方自定义消息(type = 100)
     * //第三方定义的body体，json格式
     * {
     * "myKey":myValue
     * }
     *
     * @required
     */
    private String body;

    /**
     * 对于对接了易盾反垃圾功能的应用，本消息是否需要指定经由易盾检测的内容（antispamCustom）。
     * true或false, 默认false。
     * 只对消息类型为：100 自定义消息类型 的消息生效。
     */
    private String antispam;

    /**
     * 在antispam参数为true时生效。
     * 自定义的反垃圾检测内容, JSON格式，长度限制同body字段，不能超过5000字符，要求antispamCustom格式如下：
     * <p>
     * {"type":1,"data":"custom content"}
     * <p>
     * 字段说明：
     * 1. type: 1：文本，2：图片。
     * 2. data: 文本内容or图片地址。
     */
    private String antispamCustom;

    /**
     * 发消息时特殊指定的行为选项,JSON格式，可用于指定消息的漫游，存云端历史，发送方多端同步，推送，消息抄送等特殊行为;option中字段不填时表示默认值 ，option示例:
     * <p>
     * {"push":false,"roam":true,"history":false,"sendersync":true,"route":false,"badge":false,"needPushNick":true}
     * <p>
     * 字段说明：
     * 1. roam: 该消息是否需要漫游，默认true（需要app开通漫游消息功能）；
     * 2. history: 该消息是否存云端历史，默认true；
     * 3. sendersync: 该消息是否需要发送方多端同步，默认true；
     * 4. push: 该消息是否需要APNS推送或安卓系统通知栏推送，默认true；
     * 5. route: 该消息是否需要抄送第三方；默认true (需要app开通消息抄送功能);
     * 6. badge:该消息是否需要计入到未读计数中，默认true;
     * 7. needPushNick: 推送文案是否需要带上昵称，不设置该参数时默认true;
     * 8. persistent: 是否需要存离线消息，不设置该参数时默认true。
     */
    private String option;

    /**
     * 推送文案,最长500个字符，android以此为推送显示文案；ios若未填写payload，显示文案以pushcontent为准。
     */
    private String pushcontent;

    /**
     * 推送对应的payload,必须是JSON,不能超过2k字符。
     */
    private String payload;

    /**
     * 开发者扩展字段，长度限制1024字符
     */
    private String ext;

    /**
     * 发送群消息时的强推用户列表（云信demo中用于承载被@的成员），格式为JSONArray，如["accid1","accid2"]。若forcepushall为true，则forcepushlist为除发送者外的所有有效群成员
     */
    @JSONField(name = "forcePushList")
    private String forcepushlist;

    /**
     * 发送群消息时，针对强推列表forcepushlist中的用户，强制推送的内容
     */
    @JSONField(name = "forcepushcontent")
    private String forcePushContent;

    /**
     * 发送群消息时，强推列表是否为群里除发送者外的所有有效成员，true或false，默认为false
     */
    @JSONField(name = "forcepushall")
    private String forcePushAll;

    /**
     * 可选，反垃圾业务ID，实现“单条消息配置对应反垃圾”，若不填则使用原来的反垃圾配置
     */
    private String bid;

    /**
     * 可选，单条消息是否使用易盾反垃圾，可选值为0。
     * 0：（在开通易盾的情况下）不使用易盾反垃圾而是使用通用反垃圾，包括自定义消息。
     * 若不填此字段，即在默认情况下，若应用开通了易盾反垃圾功能，则使用易盾反垃圾来进行垃圾消息的判断
     */
    private int useYidun;

    /**
     * 可选，群消息是否需要已读业务（仅对群消息有效），0:不需要，1:需要
     */
    private int markRead;

    /**
     * 是否为好友关系才发送消息，默认否
     * 注：使用该参数需要先开通功能服务
     */
    private boolean checkFriend;


    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.SEND_MSG;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("from", from);
            put("ope", ope);
            put("to", to);
            put("type", type);
            put("body", body);
            put("antispam", antispam);
            put("antispamCustom", antispamCustom);
            put("option", option);
            put("pushcontent", pushcontent);
            put("payload", payload);
            put("ext", ext);
            put("forcepushlist", forcepushlist);
            put("forcepushcontent", forcePushContent);
            put("forcepushall", forcePushAll);
            put("bid", bid);
            put("useYidun", useYidun);
            put("markRead", markRead);
            put("checkFriend", checkFriend);
        }};
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseMsgSendResponse> getResponseClass() {
        return NeteaseMsgSendResponse.class;
    }
}
