package com.starbuds.server.service.message.netease.request.msg;

import com.alibaba.fastjson.annotation.JSONField;
import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.response.msg.NeteaseMsgUploadResponse;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/11 13:44
 * @Description 文件上传
 * 文件上传，字符流需要base64编码，最大15M。
 */
@Data
public class NeteaseMsgUploadRequest implements NeteaseRequest<NeteaseMsgUploadResponse> {

    /**
     * 字符流base64串(Base64.encode(bytes)) ，最大15M的字符流
     *
     * @required
     */
    private String content;

    /**
     * 上传文件类型
     */
    private String type;

    /**
     * 返回的url是否需要为https的url，true或false，默认false
     */
    @JSONField(name = "ishttps")
    private String isHttps;

    /**
     * 文件过期时长，单位：秒，必须大于等于86400
     */
    private Integer expireSec;

    /**
     * 文件的应用场景，不超过32个字符
     */
    private String tag;


    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.MSG_UPLOAD;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("content", content);
            put("type", type);
            put("ishttps", isHttps);
            put("expireSec", expireSec);
            put("tag", tag);
        }};
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseMsgUploadResponse> getResponseClass() {
        return NeteaseMsgUploadResponse.class;
    }
}
