package com.starbuds.server.service.message.netease.request.team;

import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.response.team.NeteaseTeamAddResponse;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/11 15:07
 * @Description 群组功能（高级群） - 拉人入群
 */
@Data
public class NeteaseTeamAddRequest implements NeteaseRequest<NeteaseTeamAddResponse> {

    /**
     * 网易云通信服务器产生，群唯一标识，创建群时会返回，最大长度128字符
     *
     * @required
     */
    private String tid;

    /**
     * 群主用户帐号，最大长度32字符
     *
     * @required
     */
    private String owner;

    /**
     * ["aaa","bbb"](JSONArray对应的accid，如果解析出错会报414)，一次最多拉200个成员
     *
     * @required
     */
    private String members;

    /**
     * 管理后台建群时，0不需要被邀请人同意加入群，1需要被邀请人同意才可以加入群。其它会返回414
     *
     * @required
     */
    private int magree;

    /**
     * 邀请发送的文字，最大长度150字符
     *
     * @required
     */
    private String msg;

    /**
     * 自定义扩展字段，最大长度512
     */
    private String attach;

    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.TEAM_ADD;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("tid", tid);
            put("owner", owner);
            put("members", members);
            put("magree", magree);
            put("msg", msg);
            put("attach", attach);
        }};
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseTeamAddResponse> getResponseClass() {
        return NeteaseTeamAddResponse.class;
    }
}
