package com.starbuds.server.service.message.netease.request.team;

import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.response.team.NeteaseTeamKickResponse;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/11 15:14
 * @Description 踢人出群
 */
@Data
public class NeteaseTeamKickRequest implements NeteaseRequest<NeteaseTeamKickResponse> {

    /**
     * 网易云通信服务器产生，群唯一标识，创建群时会返回，最大长度128字符
     *
     * @required
     */
    private String tid;

    /**
     * 群主的accid，用户帐号，最大长度32字符
     *
     * @required
     */
    private String owner;

    /**
     * 被移除人的accid，用户账号，最大长度32字符;注：member或members任意提供一个，优先使用member参数
     */
    private String member;

    /**
     * ["aaa","bbb"]（JSONArray对应的accid，如果解析出错，会报414）一次最多操作200个accid; 注：member或members任意提供一个，优先使用member参数
     */
    private String members;

    /**
     * 自定义扩展字段，最大长度512
     */
    private String attach;


    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.TEAM_KICK;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("tid", tid);
            put("owner", owner);
            put("member", member);
            put("members", members);
            put("attach", attach);
        }};
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseTeamKickResponse> getResponseClass() {
        return NeteaseTeamKickResponse.class;
    }
}
