package com.starbuds.server.service.message.netease.request.team;

import com.alibaba.fastjson.annotation.JSONField;
import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.response.team.NeteaseTeamMarkReadInfoResponse;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/11 17:44
 * @Description 获取群组已读消息的已读详情信息
 */
@Data
public class NeteaseTeamMarkReadInfoRequest implements NeteaseRequest<NeteaseTeamMarkReadInfoResponse> {

    /**
     * 群id，群唯一标识，创建群时会返回
     *
     * @required
     */
    private long tid;
    /**
     * 发送群已读业务消息时服务器返回的消息ID
     *
     * @required
     */
    @JSONField(name = "msgid")
    private long msgId;
    /**
     * 消息发送者账号
     *
     * @required
     */
    @JSONField(name = "fromAccid")
    private String fromAccId;
    /**
     * 是否返回已读、未读成员的accid列表，默认为false
     */
    private Boolean snapshot;

    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.TEAM_MARK_READ_INFO;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("tid", tid);
            put("msgid", msgId);
            put("fromAccid", fromAccId);
            put("snapshot", snapshot);
        }};
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseTeamMarkReadInfoResponse> getResponseClass() {
        return NeteaseTeamMarkReadInfoResponse.class;
    }
}
