package com.starbuds.server.service.message.netease.request.team;

import com.alibaba.fastjson.annotation.JSONField;
import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.response.team.NeteaseTeamUpdateNickResponse;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/11 19:32
 * @Description 修改群昵称
 */
@Data
public class NeteaseTeamUpdateNickRequest implements NeteaseRequest<NeteaseTeamUpdateNickResponse> {

    /**
     * 群唯一标识，创建群时网易云通信服务器产生并返回
     *
     * @required
     */
    private String tid;

    /**
     * 群主 accid
     *
     * @required
     */
    private String owner;

    /**
     * 要修改群昵称的群成员 accid
     *
     * @required
     */
    @JSONField(name = "accid")
    private String accId;

    /**
     * accid 对应的群昵称，最大长度32字符
     */
    private String nick;

    /**
     * 自定义扩展字段，最大长度1024字节
     */
    private String custom;

    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.TEAM_UPDATE_NICK;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("tid", tid);
            put("owner", owner);
            put("accid", accId);
            put("nick", nick);
            put("custom", custom);
        }};
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseTeamUpdateNickResponse> getResponseClass() {
        return NeteaseTeamUpdateNickResponse.class;
    }
}
