package com.starbuds.server.service.message.netease.request.team;

import com.alibaba.fastjson.annotation.JSONField;
import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.response.team.NeteaseTeamUpdateResponse;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/11 16:08
 * @Description 编辑群资料
 */
@Data
public class NeteaseTeamUpdateRequest implements NeteaseRequest<NeteaseTeamUpdateResponse> {


    /**
     * 网易云通信服务器产生，群唯一标识，创建群时会返回
     *
     * @required
     */
    private String tid;

    /**
     * 群名称，最大长度64字符
     */
    @JSONField(name = "tname")
    private String tName;

    /**
     * 群主用户帐号，最大长度32字符
     *
     * @required
     */
    private String owner;

    /**
     * 群公告，最大长度1024字符
     */
    private String announcement;

    /**
     * 群描述，最大长度512字符
     */
    private String intro;

    /**
     * 群建好后，sdk操作时，0不用验证，1需要验证,2不允许任何人加入。其它返回414
     */
    @JSONField(name = "joinmode")
    private int joinMode;

    /**
     * 自定义高级群扩展属性，第三方可以跟据此属性自定义扩展自己的群属性。（建议为json）,最大长度1024字符
     */
    private String custom;

    /**
     * 群头像，最大长度1024字符
     */
    private String icon;

    /**
     * 被邀请人同意方式，0-需要同意(默认),1-不需要同意。其它返回414
     */
    @JSONField(name = "beinvitemode")
    private int beInviteMode;

    /**
     * 谁可以邀请他人入群，0-管理员(默认),1-所有人。其它返回414
     */
    @JSONField(name = "invitemode")
    private int inviteMode;

    /**
     * 谁可以修改群资料，0-管理员(默认),1-所有人。其它返回414
     */
    @JSONField(name = "uptinfomode")
    private int uptInfoMode;

    /**
     * 谁可以更新群自定义属性，0-管理员(默认),1-所有人。其它返回414
     */
    @JSONField(name = "upcustommode")
    private int upCustomMode;

    /**
     * 该群最大人数(包含群主)，范围：2至应用定义的最大群人数(默认:200)。其它返回414
     */
    private int teamMemberLimit;

    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.TEAM_UPDATE;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("tid", tid);
            put("tname", tName);
            put("owner", owner);
            put("announcement", announcement);
            put("intro", intro);
            put("joinmode", joinMode);
            put("custom", custom);
            put("icon", icon);
            put("beinvitemode", beInviteMode);
            put("invitemode", inviteMode);
            put("uptinfomode", uptInfoMode);
            put("upcustommode", upCustomMode);
            put("teamMemberLimit", teamMemberLimit);
        }};
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseTeamUpdateResponse> getResponseClass() {
        return NeteaseTeamUpdateResponse.class;
    }
}
