package com.starbuds.server.service.message.netease.request.user;

import com.alibaba.fastjson.annotation.JSONField;
import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.response.user.NeteaseUserAddFriendResponse;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/10 17:42
 * @Description 加好友
 */
@Data
public class NeteaseUserAddFriendRequest implements NeteaseRequest<NeteaseUserAddFriendResponse> {

    /**
     * 加好友发起者accid
     *
     * @required
     */
    @JSONField(name = "accid")
    private String accId;
    /**
     * 加好友接收者accid
     *
     * @required
     */
    @JSONField(name = "faccid")
    private String faccId;
    /**
     * 1直接加好友，2请求加好友，3同意加好友，4拒绝加好友
     *
     * @required
     */
    private int type;
    /**
     * 加好友对应的请求消息，第三方组装，最长256字符
     */
    private String msg;
    /**
     * 服务器端扩展字段，限制长度256
     * 此字段client端只读，server端读写
     */
    @JSONField(name = "serverex")
    private String serverEx;

    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.ADD_FRIEND;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("accid", accId);
            put("faccid", faccId);
            put("type", type);
            put("msg", msg);
            put("serverex", serverEx);
        }};
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseUserAddFriendResponse> getResponseClass() {
        return NeteaseUserAddFriendResponse.class;
    }
}
