package com.starbuds.server.service.message.netease.request.user;

import com.alibaba.fastjson.annotation.JSONField;
import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.response.user.NeteaseUserBlockResponse;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/14 14:24
 * @Description 封禁网易云通信ID
 * 接口描述
 * 1.封禁网易云通信ID后，此ID将不能再次登录。若封禁时，该id处于登录状态，则当前登录不受影响，仍然可以收发消息。封禁效果会在下次登录时生效。因此建议，将needkick设置为true，让该账号同时被踢出登录。
 * 2.封禁时踢出，会触发登出事件消息抄送。
 * 3.出于安全目的，账号创建后只能封禁，不能删除；封禁后账号仍计入应用内账号总数。
 */
@Data
public class NeteaseUserBlockRequest implements NeteaseRequest<NeteaseUserBlockResponse> {


    /**
     * 网易云通信ID，最大长度32字符，必须保证一个APP内唯一
     *
     * @required
     */
    @JSONField(name = "accid")
    private String accId;

    /**
     * 是否踢掉被禁用户，true或false，默认false
     */
    @JSONField(name = "needkick")
    private String needKick;

    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.USER_BLOCK;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("accid", accId);
            put("needkick", needKick);
        }};
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseUserBlockResponse> getResponseClass() {
        return NeteaseUserBlockResponse.class;
    }
}
