package com.starbuds.server.service.message.netease.request.user;

import com.alibaba.fastjson.annotation.JSONField;
import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import com.starbuds.server.service.message.netease.response.user.NeteaseUserSetSpecialRelationResponse;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/10 18:12
 * @Description 设置黑名单/静音
 */
@Data
public class NeteaseUserSetSpecialRelationRequest implements NeteaseRequest<NeteaseUserSetSpecialRelationResponse> {

    /**
     * 用户帐号，最大长度32字符，必须保证一个APP内唯一
     *
     * @required
     */
    @JSONField(name = "accid")
    private String accId;

    /**
     * 被加黑或加静音的帐号
     *
     * @required
     */
    private String targetAcc;

    /**
     * 本次操作的关系类型,1:黑名单操作，2:静音列表操作
     *
     * @required
     */
    private int relationType;

    /**
     * 操作值，0:取消黑名单或静音，1:加入黑名单或静音
     *
     * @required
     */
    private int value;

    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.SET_SPECIAL_RELATION;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("accid", accId);
            put("targetAcc", targetAcc);
            put("relationType", relationType);
            put("value", value);
        }};
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseUserSetSpecialRelationResponse> getResponseClass() {
        return NeteaseUserSetSpecialRelationResponse.class;
    }
}
