package com.starbuds.server.service.message.netease.request.user;

import com.alibaba.fastjson.annotation.JSONField;
import com.starbuds.server.service.message.netease.NeteaseRequest;
import com.starbuds.server.service.message.netease.NeteaseResponse;
import com.starbuds.server.service.message.netease.constant.NeteaseApiUrl;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/10 16:56
 * @Description 更新用户名片
 */
@Data
public class NeteaseUserUpdateUinfoRequest implements NeteaseRequest<NeteaseResponse> {

    /**
     * 用户帐号，最大长度32字符，必须保证一个APP内唯一、
     *
     * @required
     */
    @JSONField(name = "accid")
    private String accId;
    /**
     * 用户昵称，最大长度64字符，可设置为空字符串
     */
    private String name;
    /**
     * 用户头像，最大长度1024字节，可设置为空字符串
     */
    private String icon;
    /**
     * 用户签名，最大长度256字符，可设置为空字符串
     */
    private String sign;
    /**
     * 用户email，最大长度64字符，可设置为空字符串
     */
    private String email;
    /**
     * 用户生日，最大长度16字符，可设置为空字符串
     */
    private String birth;
    /**
     * 用户mobile，最大长度32字符，非中国大陆手机号码需要填写国家代码(如美国：+1-xxxxxxxxxx)或地区代码(如香港：+852-xxxxxxxx)，可设置为空字符串
     */
    private String mobile;
    /**
     * 用户性别，0表示未知，1表示男，2女表示女，其它会报参数错误
     */
    private int gender;
    /**
     * 用户名片扩展字段，最大长度1024字符，用户可自行扩展，建议封装成JSON字符串，也可以设置为空字符串
     */
    private String ex;

    @Override
    public String getApiUrl() {
        return NeteaseApiUrl.UPDATE_UINFO;
    }

    @Override
    public Map<String, Object> getParamsMap() {
        return new HashMap<String, Object>() {{
            put("accid", accId);
            put("name", name);
            put("icon", icon);
            put("sign", sign);
            put("email", email);
            put("birth", birth);
            put("mobile", mobile);
            put("gender", gender);
            put("ex", ex);
        }};
    }

    /**
     * 获取响应类
     *
     * @return
     */
    @Override
    public Class<NeteaseResponse> getResponseClass() {
        return NeteaseResponse.class;
    }
}
