package com.starbuds.server.service.message.provider;

import com.alibaba.fastjson.JSON;
import com.aliyuncs.CommonRequest;
import com.aliyuncs.CommonResponse;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.IAcsClient;
import com.aliyuncs.afs.model.v20180112.AuthenticateSigRequest;
import com.aliyuncs.afs.model.v20180112.AuthenticateSigResponse;
import com.aliyuncs.http.MethodType;
import com.aliyuncs.profile.DefaultProfile;
import com.aliyuncs.profile.IClientProfile;
import com.starbuds.server.common.pojo.message.SmsCode;
import com.starbuds.server.common.pojo.message.VoiceSmsCode;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/2 11:44
 * @Description 阿里云短信服务
 */
@Slf4j
@Component
public class AliyunSmsProvider implements ISmsProvider {

    public static final String appKey = "FFFF0N00000000009342";


    @Override
    public String sendSms(String templateCode, String phone, SmsCode tempParam, String accessKey, String accessSecret, String signName) {

        DefaultProfile profile = DefaultProfile.getProfile("cn-hangzhou", accessKey, accessSecret);
        IAcsClient client = new DefaultAcsClient(profile);

        CommonRequest request = new CommonRequest();
        request.setSysMethod(MethodType.POST);
        request.setSysDomain("dysmsapi.aliyuncs.com");
        request.setSysVersion("2017-05-25");
        request.setSysAction("SendSms");
        request.putQueryParameter("RegionId", "cn-hangzhou");
        request.putQueryParameter("TemplateCode", templateCode);
        request.putQueryParameter("SignName", signName);
        request.putQueryParameter("PhoneNumbers", phone);
        request.putQueryParameter("TemplateParam", JSON.toJSONString(tempParam));
        request.putQueryParameter("OutId", "smsId");

        try {
            CommonResponse response = client.getCommonResponse(request);
            log.info("发送短信验证码结果:{}", response.getData());
            return response.getData();
        } catch (Exception e) {
            log.error("发送短信验证码失败", e);
            return null;
        }
    }


    @Override
    public String sendVoiceSms(String templateCode, String calledNumber, String showNumber, VoiceSmsCode tempParam, String accessKey, String accessSecret) {

        DefaultProfile profile = DefaultProfile.getProfile("cn-hangzhou", accessKey, accessSecret);
        IAcsClient client = new DefaultAcsClient(profile);

        CommonRequest request = new CommonRequest();
        request.setMethod(MethodType.POST);
        request.setDomain("dyvmsapi.aliyuncs.com");
        request.setVersion("2017-05-25");
        request.setAction("SingleCallByTts");
        request.putQueryParameter("RegionId", "cn-hangzhou");
        request.putQueryParameter("CalledShowNumber", showNumber);
        request.putQueryParameter("CalledNumber", calledNumber);
        request.putQueryParameter("TtsParam", JSON.toJSONString(tempParam));
        request.putQueryParameter("TtsCode", templateCode);
        request.putQueryParameter("PlayTimes", "3");
        request.putQueryParameter("Volume", "60");

        try {
            CommonResponse response = client.getCommonResponse(request);
            log.info("发送短信验证码结果:{}", response.getData());
            return response.getData();
        } catch (Exception e) {
            log.error("发送短信验证码失败", e);
            return null;
        }
    }

    @Override
    public boolean machineValidate(String accessKey, String accessSecret, String sessionId, String sig, String token, String scene, String remoteIp) {
        log.info("accessKey:{},accessSecret:{}", accessKey, accessSecret);
        try {
            IClientProfile profile = DefaultProfile.getProfile("cn-hangzhou", accessKey, accessSecret);
            IAcsClient client = new DefaultAcsClient(profile);
            DefaultProfile.addEndpoint("cn-hangzhou", "cn-hangzhou", "afs", "afs.aliyuncs.com");
            AuthenticateSigRequest request = new AuthenticateSigRequest();
            request.setSessionId(sessionId);
            request.setSig(sig);
            request.setToken(token);
            request.setScene(scene);
            request.setAppKey(appKey);
            request.setRemoteIp(remoteIp);
            AuthenticateSigResponse response = client.getAcsResponse(request);
            log.info("response：{}", JSON.toJSONString(response));
            if (response != null && response.getCode() == 100) {
                return true;
            }
        } catch (Exception e) {
            log.info("人机校验出错", e);
            return false;
        }
        return false;
    }

    public static void main(String[] args) {
        DefaultProfile profile = DefaultProfile.getProfile("cn-hangzhou", "LTAIuQNXdXnGBeGN", "whRY7VAyLJOAd7aQHnuVDp4fU4714r");
        IAcsClient client = new DefaultAcsClient(profile);
        CommonRequest request = new CommonRequest();
        request.setMethod(MethodType.POST);
        request.setDomain("dyvmsapi.aliyuncs.com");
        request.setVersion("2017-05-25");
        request.setAction("SingleCallByTts");
        request.putQueryParameter("RegionId", "cn-hangzhou");
        request.putQueryParameter("CalledShowNumber", "055162143123");
        request.putQueryParameter("CalledNumber", "15157390363");
        request.putQueryParameter("TtsParam", JSON.toJSONString(new VoiceSmsCode("芝麻乐", "6789")));
        request.putQueryParameter("TtsCode", "TTS_193020521");
        request.putQueryParameter("PlayTimes", "3");
        request.putQueryParameter("Volume", "60");
        try {
            CommonResponse response = client.getCommonResponse(request);
            System.out.println(response.getData());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
