package com.starbuds.server.service.message.provider;

import com.starbuds.server.common.pojo.message.ImUserCreateDTO;
import com.starbuds.server.common.pojo.message.ImUserProfile;
import com.starbuds.server.common.pojo.message.LiveChatRoomUserProfile;
import com.starbuds.server.common.pojo.message.im.BaseImMsg;
import com.starbuds.server.common.pojo.message.im.ImPushInfo;
import com.starbuds.server.common.pojo.user.UserSimpleInfo;
import com.starbuds.server.service.message.tencent.response.biz.TencentImBizGetAppInfoResponse;
import io.rong.messages.BaseMessage;

import java.util.List;
import java.util.Map;

/**
 * IM 服务
 *
 * @author Lay
 * @date 2020/1/4
 */
public interface IImProvider {

    //---以下为老版腾讯im的封装接口---

    /**
     * 创建 IM 用户
     *
     * @param imUserCreateDTO
     * @return
     */
    String createUser(ImUserCreateDTO imUserCreateDTO);

    /**
     * 刷新 Token
     *
     * @param userId
     * @param expireTime
     * @return
     */
    String refreshToken(Long userId, Long expireTime);

    /**
     * 创建直播间聊天室
     *
     * @param userId
     * @param roomId
     * @return
     */
    String addLiveChatRoom(Long userId, String roomId);

    /**
     * 同步用户资料
     *
     * @param imProfile
     * @return
     */
    boolean syncUserProfile(ImUserProfile imProfile);

    /**
     * 发送自定义消息
     *
     * @param senderId
     * @param targetId
     * @param imMsg
     * @param imPushInfo
     * @return
     */
    boolean sendCustomImMsg(Long senderId, Long targetId, BaseImMsg imMsg, ImPushInfo imPushInfo);

    /**
     * 发送直播间消息
     *
     * @param anchorUserId
     * @param fromUserId
     * @param imMsg
     * @param msgPriority
     * @return
     */
    boolean sendLiveRoomMsg(Long anchorUserId, Long fromUserId, BaseImMsg imMsg, String msgPriority);

    /**
     * 发送广播消息
     *
     * @param channelId
     * @param fromUserId
     * @param imMsg
     * @param msgPriority
     * @return
     */
    boolean sendBroadcast(Long channelId, Long fromUserId, BaseImMsg imMsg, String msgPriority);

    /**
     * 同步群成员资料
     *
     * @param anchorUserId
     * @param imGroupUserProfile
     * @return
     */
    boolean syncLiveChatRoomUserProfile(Long anchorUserId, LiveChatRoomUserProfile imGroupUserProfile);

    /**
     * 获取app信息
     *
     * @return
     */
    TencentImBizGetAppInfoResponse getAppInfo();

    /**
     * 发送文本消息
     *
     * @param senderId
     * @param targetId
     * @param text
     * @return
     */
    boolean sendTextMsg(Long senderId, Long targetId, String text);

    //---以下为新版融云im的封装接口---

    /**
     * 创建token
     *
     * @param userId
     * @return
     */
    String buildToken(Long userId);

    /**
     * 创建token
     *
     * @param imUserProfile
     * @return
     */
    String buildToken(ImUserProfile imUserProfile);

    /**
     * 刷新用户资料
     *
     * @param imUserProfile
     * @return
     */
    boolean refreshUser(ImUserProfile imUserProfile);

    /**
     * 发送私聊信息给用户
     *
     * @param userId
     * @param targetId
     * @param content
     * @return
     */
    boolean sendToPeer(Long userId, Long targetId, String content);

    /**
     * 发送私聊信息给用户
     *
     * @param userId
     * @param targetId
     * @param data
     * @return
     */
    boolean sendToPeerCustomMsg(Long userId, Long targetId, String data);

    /**
     * 发送私聊信息给用户
     *
     * @param userId
     * @param targetId
     * @param msg
     * @param pushContent
     * @param isIncludeSender
     * @return
     */
    boolean sendToPeerCustomMsg(Long userId, Long targetId, BaseMessage msg, String pushContent, Integer isIncludeSender);

    /**
     * 发送私聊信息给用户
     *
     * @param userId
     * @param targetId
     * @param text
     * @param extra
     * @return
     */
    boolean sendToPeerTextMsg(Long userId, Long targetId, String text, String extra);

    /**
     * 发送房间消息
     *
     * @param userId
     * @param roomId
     * @param content
     * @return
     */
    boolean sendToChatRoom(Long userId, String roomId, String content);

    /**
     * 发送频道消息
     *
     * @param userId
     * @param roomId
     * @param content
     * @param lowPriority
     * @return
     */
    boolean sendToChannel(Long userId, Long roomId, String content, boolean lowPriority);

    /**
     * 聊天室禁言
     *
     * @param roomId
     * @param userId
     * @param minutes
     * @return
     */
    boolean forbiddenChat(Long roomId, Long userId, Integer minutes);

    /**
     * 解除禁言
     *
     * @param roomId
     * @param userId
     * @return
     */
    boolean enableChat(Long roomId, Long userId);

    /**
     * 封禁用户
     *
     * @param roomId
     * @param userId
     * @param minutes
     * @return
     */
    boolean blockRoomUser(Long roomId, Long userId, Integer minutes);

    /**
     * 解封用户
     *
     * @param roomId
     * @param userId
     * @return
     */
    boolean unblockRoomUser(Long roomId, Long userId);

    /**
     * 设置聊天室属性
     *
     * @param roomId
     * @param key
     * @param value
     * @return
     */
    boolean setRoomProperty(Long roomId, String key, String value);

    /**
     * 设置聊天室属性
     *
     * @param roomId
     * @param propertyMap
     */
    void setRoomProperties(Long roomId, Map<String, String> propertyMap);

    /**
     * 拉黑用户
     *
     * @param userId
     * @param targetId
     * @return
     */
    boolean addUserBlack(Long userId, Long targetId);

    /**
     * 解除拉黑
     *
     * @param userId
     * @param targetId
     * @return
     */
    boolean removeUserBlack(Long userId, Long targetId);

    /**
     * 获取拉黑名单
     *
     * @param userId
     * @return
     */
    List<String> getUserBlack(Long userId);

    /**
     * 私聊禁言
     *
     * @param userId
     * @return
     */
    boolean forbiddenWords(Long userId);

    /**
     * 取消私聊禁言
     *
     * @param userId
     * @return
     */
    boolean cancelForbiddenWords(Long userId);

    /**
     * 获取融云历史消息
     *
     * @param date
     * @return
     */
    String getHistory(String date);

    /**
     * 获取用户信息
     *
     * @param userId
     * @return
     */
    UserSimpleInfo getUserInfo(Long userId);

    /**
     * 对比签名
     *
     * @param timestamp
     * @param nonce
     * @param sign
     * @return
     */
    boolean checkNotifySign(String timestamp, String nonce, String sign);

    /**
     * 获取房间内在线用户id
     *
     * @param roomId
     * @return
     * @throws Exception
     */
    List<Long> onlineUserIdsInRoom(Long roomId) throws Exception;
}
