package com.starbuds.server.service.message.provider;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.RandomUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.user.LiveUserRole;
import com.starbuds.server.common.pojo.message.ImOnlineStatus;
import com.starbuds.server.common.pojo.message.ImUserCreateDTO;
import com.starbuds.server.common.pojo.message.ImUserProfile;
import com.starbuds.server.common.pojo.message.LiveChatRoomUserProfile;
import com.starbuds.server.common.pojo.message.im.BaseImMsg;
import com.starbuds.server.common.pojo.message.im.ImPushInfo;
import com.starbuds.server.common.pojo.message.im.rtc.RtcConveneFansMsg;
import com.starbuds.server.common.pojo.user.UserBlackDTO;
import com.starbuds.server.common.pojo.user.UserSimpleInfo;
import com.starbuds.server.service.message.tencent.TencentClientManage;
import com.starbuds.server.service.message.tencent.constant.TencentGroupType;
import com.starbuds.server.service.message.tencent.constant.TencentProfileTag;
import com.starbuds.server.service.message.tencent.entity.CustomContent;
import com.starbuds.server.service.message.tencent.entity.TencentImMsgCustomerElement;
import com.starbuds.server.service.message.tencent.entity.TencentImMsgTextElement;
import com.starbuds.server.service.message.tencent.entity.TextContent;
import com.starbuds.server.service.message.tencent.param.account.TencentImOpenLoginSvcAccountImportParam;
import com.starbuds.server.service.message.tencent.param.account.TencentOpenImQueryStateParam;
import com.starbuds.server.service.message.tencent.param.config.TencentImGetNoSpeakingParam;
import com.starbuds.server.service.message.tencent.param.config.TencentImSetNoSpeakingParam;
import com.starbuds.server.service.message.tencent.param.group.TencentImGroupCreateParam;
import com.starbuds.server.service.message.tencent.param.group.TencentImGroupModifyGroupMemberInfoParam;
import com.starbuds.server.service.message.tencent.param.group.TencentImGroupSendGroupMsgParam;
import com.starbuds.server.service.message.tencent.param.msg.TencentImMsgSendParam;
import com.starbuds.server.service.message.tencent.param.profile.TencentImProfileSetParam;
import com.starbuds.server.service.message.tencent.param.sns.*;
import com.starbuds.server.service.message.tencent.request.account.TencentImOpenLoginSvcAccountImportRequest;
import com.starbuds.server.service.message.tencent.request.account.TencentOpenImQueryStateRequest;
import com.starbuds.server.service.message.tencent.request.biz.TencentImBizGetAppInfoRequest;
import com.starbuds.server.service.message.tencent.request.config.TencentImGetNoSpeakingRequest;
import com.starbuds.server.service.message.tencent.request.config.TencentImSetNoSpeakingRequest;
import com.starbuds.server.service.message.tencent.request.group.TencentImGroupCreateRequest;
import com.starbuds.server.service.message.tencent.request.group.TencentImGroupModifyGroupMemberInfoRequest;
import com.starbuds.server.service.message.tencent.request.group.TencentImGroupSendGroupMsgRequest;
import com.starbuds.server.service.message.tencent.request.msg.TencentImMsgSendRequest;
import com.starbuds.server.service.message.tencent.request.profile.TencentImProfileSetRequest;
import com.starbuds.server.service.message.tencent.request.sns.*;
import com.starbuds.server.service.message.tencent.response.account.TencentImOpenLoginSvcAccountImportResponse;
import com.starbuds.server.service.message.tencent.response.account.TencentOpenImQueryStateResponse;
import com.starbuds.server.service.message.tencent.response.biz.TencentImBizGetAppInfoResponse;
import com.starbuds.server.service.message.tencent.response.config.TencentImGetNoSpeakingResponse;
import com.starbuds.server.service.message.tencent.response.config.TencentImSetNoSpeakingResponse;
import com.starbuds.server.service.message.tencent.response.group.TencentImGroupCreateResponse;
import com.starbuds.server.service.message.tencent.response.group.TencentImGroupModifyGroupMemberInfoResponse;
import com.starbuds.server.service.message.tencent.response.group.TencentImGroupSendGroupMsgResponse;
import com.starbuds.server.service.message.tencent.response.msg.TencentImMsgSendResponse;
import com.starbuds.server.service.message.tencent.response.profile.TencentImProfileSetResponse;
import com.starbuds.server.service.message.tencent.response.sns.*;
import com.starbuds.server.service.message.tencent.util.TLSSigAPIv2;
import io.rong.messages.BaseMessage;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.*;

/**
 * 备注
 * 腾讯im改为融云im
 *
 * @author Lay
 * @date 2020/1/3
 * @see RongCloudProvider
 */
@Deprecated
@Component
@Slf4j
public class TencentImProvider implements IImProvider {

    @Value("${im.tencent.appKey}")
    private Long appKey;

    @Value("${im.tencent.userSig}")
    private String userSig;

    @Value("${im.tencent.appSecret}")
    private String appSecret;

    @Value("${im.tencent.identifier}")
    private String identifier;

    @Resource
    TencentClientManage tencentClientManage;

    /**
     * 全员大群 ID
     */
    public static final String BCR_ID = "BCR2020";

    private static final Long expire = 90 * 24 * 3600L;

    /**
     * 初始化
     */
    public void init() {
        // 创建固定的全员大群
        TencentImGroupCreateParam param = new TencentImGroupCreateParam();
        param.setType(TencentGroupType.BCHATROOM);
        param.setName("星芽");
        param.setGroupId(BCR_ID);

        TencentImGroupCreateRequest request = new TencentImGroupCreateRequest();
        request.setParam(param);

        tencentClientManage.getClient().execute(request);
    }

    public Long getSdkAppId() {
        return tencentClientManage.getSdkAppId();
    }

    /**
     * 创建 IM 用户
     *
     * @param createDTO
     * @return
     */
    @Override
    public String createUser(ImUserCreateDTO createDTO) {
        TencentImOpenLoginSvcAccountImportParam build = TencentImOpenLoginSvcAccountImportParam.builder().Identifier(createDTO.getUserId().toString()).build();
        TencentImOpenLoginSvcAccountImportRequest request = new TencentImOpenLoginSvcAccountImportRequest();
        request.setParam(build);
        TencentImOpenLoginSvcAccountImportResponse response = tencentClientManage.getClient().execute(request);
        if (response.isSuccess()) {
            return refreshToken(createDTO.getUserId(), expire);
        }
        return null;
    }

    /**
     * 刷新 Token
     *
     * @param userId
     * @param expireTime
     * @return
     */
    @Override
    public String refreshToken(Long userId, Long expireTime) {
        TLSSigAPIv2 sign = new TLSSigAPIv2(appKey, appSecret);
        return sign.genSig(userId.toString(), expireTime);
    }

    /**
     * 创建直播间聊天室
     *
     * @param userId
     * @param roomId
     * @return
     */
    @Override
    public String addLiveChatRoom(Long userId, String roomId) {
        TencentImGroupCreateParam param = new TencentImGroupCreateParam();
        param.setOwnerAccount(String.valueOf(userId));
        param.setType(TencentGroupType.AVCHATROOM);
        param.setGroupId(roomId);
        param.setName("直播间");

        TencentImGroupCreateRequest request = new TencentImGroupCreateRequest();
        request.setParam(param);

        TencentImGroupCreateResponse response = tencentClientManage.getClient().execute(request);
        if (response.isSuccess()) {
            return response.getGroupId();
        }
        return null;
    }

    /**
     * 同步用户资料
     *
     * @param imProfile
     * @return
     */
    @Override
    public boolean syncUserProfile(ImUserProfile imProfile) {
        TencentImProfileSetParam param = new TencentImProfileSetParam();
        param.setFromAccount(String.valueOf(imProfile.getUserId()));

        List<TencentImProfileSetParam.ProfileItem> profileItems = new ArrayList<>();

        TencentImProfileSetParam.ProfileItem allowType = new TencentImProfileSetParam.ProfileItem();
        allowType.setTag(TencentProfileTag.TAG_PROFILE_IM_ALLOWTYPE);
        allowType.setValue("AllowType_Type_NeedConfirm");
        profileItems.add(allowType);

        // 昵称
        if (StringUtils.isNotBlank(imProfile.getUserName())) {
            TencentImProfileSetParam.ProfileItem profileItem = new TencentImProfileSetParam.ProfileItem();
            profileItem.setTag(TencentProfileTag.TAG_PROFILE_IM_NICK);
            profileItem.setValue(imProfile.getUserName());
            profileItems.add(profileItem);
        }

        // 头像
        if (StringUtils.isNotBlank(imProfile.getUserAvatar())) {
            TencentImProfileSetParam.ProfileItem profileItem = new TencentImProfileSetParam.ProfileItem();
            profileItem.setTag(TencentProfileTag.TAG_PROFILE_IM_IMAGE);
            profileItem.setValue(imProfile.getUserAvatar());
            profileItems.add(profileItem);
        }

        // 性别
        if (imProfile.getUserSex() != null) {
            TencentImProfileSetParam.ProfileItem profileItem = new TencentImProfileSetParam.ProfileItem();
            profileItem.setTag(TencentProfileTag.TAG_PROFILE_IM_GENDER);
            profileItem.setValue(BoolIntVal.True == imProfile.getUserSex() ? "Gender_Type_Male" : "Gender_Type_Female");
            profileItems.add(profileItem);
        }

        if (CollectionUtil.isNotEmpty(profileItems)) {
            param.setProfileItem(profileItems);
            TencentImProfileSetRequest request = new TencentImProfileSetRequest();
            request.setParam(param);
            TencentImProfileSetResponse response = tencentClientManage.getClient().execute(request);
            if (response.isSuccess()) {
                return true;
            }
        }
        return false;
    }

    /**
     * 发送自定义消息
     *
     * @param imMsg
     * @param imPushInfo
     * @return
     */
    @Override
    public boolean sendCustomImMsg(Long senderId, Long targetId, BaseImMsg imMsg, ImPushInfo imPushInfo) {
        TencentImMsgSendParam param = new TencentImMsgSendParam();
        if (senderId != null) {
            param.setFromAccount(String.valueOf(senderId));
        } else {
            // 系统消息不离线保存
            param.setMsgLifeTime(0);
        }
        // 如果是语聊召集粉丝通知  =>  不同步至 From_Account  主播会突然收到多条消息，导致内存上升崩溃
        if (imMsg instanceof RtcConveneFansMsg) {
            param.setSyncOtherMachine(2);
        }
        param.setToAccount(String.valueOf(targetId));
        param.setMsgRandom(Math.abs(RandomUtil.randomInt()));

        CustomContent customContent = new CustomContent();
        customContent.setData(JSON.toJSONString(imMsg));
        TencentImMsgCustomerElement customerElement = new TencentImMsgCustomerElement();
        customerElement.setMsgContent(customContent);

        param.setMsgBody(Arrays.asList(customerElement));

        TencentImMsgSendRequest request = new TencentImMsgSendRequest();
        request.setParam(param);
        log.info(JSON.toJSONString(request));
        TencentImMsgSendResponse response = tencentClientManage.getClient().execute(request);
        log.info(JSON.toJSONString(response));
        return response.isSuccess();
    }

    public boolean addFriend(Long userId, Long targetUserId, String sourceType, String remarks, boolean isForce) {
        TencentImSnsFriendAddParam param = new TencentImSnsFriendAddParam();
        param.setFromAccount(String.valueOf(userId));
        param.setAddType("Add_Type_Both");
        param.setForceAddFlags(isForce ? 1 : 0);

        TencentImSnsFriendAddParam.Item item = new TencentImSnsFriendAddParam.Item();
        item.setToAccount(String.valueOf(targetUserId));
        item.setAddSource(String.format("AddSource_Type_%s", sourceType));
        item.setAddWording(remarks);
        param.setAddFriendItem(Arrays.asList(item));

        TencentImSnsFriendAddRequest request = new TencentImSnsFriendAddRequest();
        request.setParam(param);

        TencentImSnsFriendAddResponse response = tencentClientManage.getClient().execute(request);
        return response.isSuccess();
    }

    public boolean delFriend(Long userId, Long targetUserId) {
        TencentImSnsFriendDeleteParam param = new TencentImSnsFriendDeleteParam();
        param.setFromAccount(String.valueOf(userId));
        param.setToAccount(Arrays.asList(String.valueOf(targetUserId)));
        param.setDeleteType("Delete_Type_Both");

        TencentImSnsFriendDeleteRequest request = new TencentImSnsFriendDeleteRequest();
        request.setParam(param);

        TencentImSnsFriendDeleteResponse response = tencentClientManage.getClient().execute(request);
        return response.isSuccess();
    }

    /**
     * 添加黑名单
     *
     * @param userId
     * @param targetUserId
     * @return
     */
    public boolean addBlack(Long userId, Long targetUserId) {
        TencentImSnsBlackListAddRequest request = new TencentImSnsBlackListAddRequest();

        TencentImSnsBlackListAddParam param = new TencentImSnsBlackListAddParam();
        param.setFromAccount(Convert.toStr(userId));
        param.setToAccount(Arrays.asList(Convert.toStr(targetUserId)));
        request.setParam(param);

        TencentImSnsBlackListAddResponse response = tencentClientManage.getClient().execute(request);
        return response.isSuccess();
    }

    /**
     * 删除黑名单
     *
     * @param userId
     * @param targetUserId
     * @return
     */
    public boolean delBlack(Long userId, Long targetUserId) {
        TencentImSnsBlackListDeleteRequest request = new TencentImSnsBlackListDeleteRequest();

        TencentImSnsBlackListDeleteParam param = new TencentImSnsBlackListDeleteParam();
        param.setFromAccount(Convert.toStr(userId));
        param.setToAccount(Arrays.asList(Convert.toStr(targetUserId)));
        request.setParam(param);

        TencentImSnsBlackListDeleteResponse response = tencentClientManage.getClient().execute(request);
        return response.isSuccess();
    }

    /**
     * 获取黑名单
     *
     * @param userId
     * @param pageIndex
     * @param pageSize
     * @param lastSequence
     * @return
     */
    public List<UserBlackDTO> getBlackList(Long userId, int pageIndex, int pageSize, Integer lastSequence) {
        TencentImSnsBlackListGetRequest request = new TencentImSnsBlackListGetRequest();

        TencentImSnsBlackListGetParam param = new TencentImSnsBlackListGetParam();
        param.setFromAccount(Convert.toStr(userId));
        param.setStartIndex((pageIndex - 1) * pageSize);
        param.setMaxLimited(pageSize);
        param.setLastSequence(lastSequence);
        request.setParam(param);

        TencentImSnsBlackListGetResponse response = tencentClientManage.getClient().execute(request);

        List<UserBlackDTO> blackDTOList = new ArrayList<>();

        if (response.isSuccess()) {
            List<TencentImSnsBlackListGetResponse.Item> itemList = response.getBlackListItem();
            if (!CollectionUtil.isEmpty(itemList)) {
                for (TencentImSnsBlackListGetResponse.Item item : itemList) {
                    UserBlackDTO blackDTO = new UserBlackDTO();
                    blackDTO.setUserId(Convert.toLong(item.getToAccount()));
                    blackDTO.setBlackTime(item.getAddBlackTimeStamp());
                    blackDTOList.add(blackDTO);
                }
            }
        }
        return blackDTOList;
    }

    /**
     * 发送直播间消息
     *
     * @param anchorUserId
     * @param fromUserId
     * @param imMsg
     * @param msgPriority
     * @return
     */
    @Override
    public boolean sendLiveRoomMsg(Long anchorUserId, Long fromUserId, BaseImMsg imMsg, String msgPriority) {
        TencentImGroupSendGroupMsgParam param = new TencentImGroupSendGroupMsgParam();
        param.setGroupId(String.format("LCR%d", anchorUserId));
        param.setRandom(Math.abs(RandomUtil.randomInt()));
        param.setMsgPriority(msgPriority);

        if (fromUserId != null) {
            param.setFromAccount(String.valueOf(fromUserId));
        }

        CustomContent customContent = new CustomContent();
        customContent.setData(JSON.toJSONString(imMsg));
        TencentImMsgCustomerElement customerElement = new TencentImMsgCustomerElement();
        customerElement.setMsgContent(customContent);
        param.setMsgBody(Arrays.asList(customerElement));

        TencentImGroupSendGroupMsgRequest request = new TencentImGroupSendGroupMsgRequest();
        request.setParam(param);

        TencentImGroupSendGroupMsgResponse response = tencentClientManage.getClient().execute(request);
        return response.isSuccess();
    }

    /**
     * 发送广播消息
     *
     * @param channelId
     * @param fromUserId
     * @param imMsg
     * @param msgPriority
     * @return
     */
    @Override
    public boolean sendBroadcast(Long channelId, Long fromUserId, BaseImMsg imMsg, String msgPriority) {
        // 默认为全渠道广播群
        channelId = Optional.ofNullable(channelId).map(t -> t > 0 ? t : 2020L).orElse(2020L);

        TencentImGroupSendGroupMsgParam param = new TencentImGroupSendGroupMsgParam();
        param.setGroupId(String.format("BCR%d", channelId));
        param.setRandom(Math.abs(RandomUtil.randomInt()));
        param.setMsgPriority(msgPriority);

        if (fromUserId != null) {
            param.setFromAccount(String.valueOf(fromUserId));
        }

        CustomContent customContent = new CustomContent();
        customContent.setData(JSON.toJSONString(imMsg));
        TencentImMsgCustomerElement customerElement = new TencentImMsgCustomerElement();
        customerElement.setMsgContent(customContent);
        param.setMsgBody(Arrays.asList(customerElement));

        TencentImGroupSendGroupMsgRequest request = new TencentImGroupSendGroupMsgRequest();
        request.setParam(param);

        TencentImGroupSendGroupMsgResponse response = tencentClientManage.getClient().execute(request);
        return response.isSuccess();
    }

    /**
     * 同步群成员资料
     *
     * @param anchorUserId
     * @param liveChatRoomUserProfile
     * @return
     */
    @Override
    public boolean syncLiveChatRoomUserProfile(Long anchorUserId, LiveChatRoomUserProfile liveChatRoomUserProfile) {
        // 同步资料
        TencentImGroupModifyGroupMemberInfoParam modifyGroupMemberInfoParam = new TencentImGroupModifyGroupMemberInfoParam();
        modifyGroupMemberInfoParam.setGroupId(String.format("LCR%d", anchorUserId));
        modifyGroupMemberInfoParam.setMemberAccount(String.valueOf(liveChatRoomUserProfile.getUserId()));
        modifyGroupMemberInfoParam.setNameCard(liveChatRoomUserProfile.getProfileString());
        String role = "Member";
        if (LiveUserRole.ANCHOR == liveChatRoomUserProfile.getLiveRole() ||
                LiveUserRole.MASTER == liveChatRoomUserProfile.getLiveRole() ||
                LiveUserRole.SUPER == liveChatRoomUserProfile.getLiveRole()) {
            role = "Admin";
        }
        modifyGroupMemberInfoParam.setRole(role);

        TencentImGroupModifyGroupMemberInfoRequest modifyGroupMemberInfoRequest = new TencentImGroupModifyGroupMemberInfoRequest();
        modifyGroupMemberInfoRequest.setParam(modifyGroupMemberInfoParam);

        TencentImGroupModifyGroupMemberInfoResponse response = tencentClientManage.getClient().execute(modifyGroupMemberInfoRequest);
        return response.isSuccess();
    }

    @Override
    public TencentImBizGetAppInfoResponse getAppInfo() {
        TencentImBizGetAppInfoRequest getAppInfoRequest = new TencentImBizGetAppInfoRequest();
        TencentImBizGetAppInfoResponse response = tencentClientManage.getClient().execute(getAppInfoRequest);
        return response;
    }

    /**
     * 发送文本消息
     *
     * @param senderId
     * @param targetId
     * @param text
     * @return
     */
    @Override
    public boolean sendTextMsg(Long senderId, Long targetId, String text) {
        TencentImMsgSendRequest request = new TencentImMsgSendRequest();

        TencentImMsgSendParam param = new TencentImMsgSendParam();
        param.setFromAccount(Convert.toStr(senderId));
        param.setSyncOtherMachine(1);
        param.setToAccount(Convert.toStr(targetId));
        param.setMsgRandom(Math.abs(RandomUtil.randomInt()));

        TextContent textContent = new TextContent();
        textContent.setText(text);
        TencentImMsgTextElement textElement = new TencentImMsgTextElement();
        textElement.setMsgContent(textContent);
        param.setMsgBody(Arrays.asList(textElement));
        request.setParam(param);

        log.info(JSON.toJSONString(request));
        TencentImMsgSendResponse response = tencentClientManage.getClient().execute(request);
        log.info(JSON.toJSONString(response));
        return response.isSuccess();

    }

    /**
     * 获取在线状态
     *
     * @param accountIds
     * @return
     */
    public List<ImOnlineStatus> getOnlineStatus(List<String> accountIds, boolean isNeedDetail) {
        TencentOpenImQueryStateRequest request = new TencentOpenImQueryStateRequest();
        TencentOpenImQueryStateParam param = new TencentOpenImQueryStateParam();
        param.setToAccount(accountIds);
        param.setIsNeedDetail(BoolIntVal.fromBool(isNeedDetail));
        request.setParam(param);

        TencentOpenImQueryStateResponse response = tencentClientManage.getClient().execute(request);
        if (response.isSuccess()) {
            List<ImOnlineStatus> list = new ArrayList<>();
            List<TencentOpenImQueryStateResponse.QueryResult> queryResults = response.getQueryResult();
            for (TencentOpenImQueryStateResponse.QueryResult queryResult : queryResults) {
                ImOnlineStatus imOnlineStatus = new ImOnlineStatus();
                imOnlineStatus.setUserId(Convert.toLong(queryResult.getToAccount()));
                imOnlineStatus.setOnlineStatus(queryResult.getState());
                list.add(imOnlineStatus);
            }
            return list;
        }
        return Collections.emptyList();
    }

    /**
     * 发送群组消息
     *
     * @param groupId
     * @param fromAccount
     * @return
     */
    public boolean sendChatGroupMsg(String groupId, String fromAccount, BaseImMsg imMsg) {
        TencentImGroupSendGroupMsgRequest request = new TencentImGroupSendGroupMsgRequest();

        TencentImGroupSendGroupMsgParam msgParam = new TencentImGroupSendGroupMsgParam();
        msgParam.setGroupId(groupId);
        msgParam.setFromAccount(fromAccount);
        msgParam.setRandom(RandomUtil.randomInt(0, 100000000));

        TencentImMsgCustomerElement customerElement = new TencentImMsgCustomerElement();
        CustomContent customContent = new CustomContent();
        customContent.setData(JSON.toJSONString(imMsg));
        customerElement.setMsgContent(customContent);
        msgParam.setMsgBody(Collections.singletonList(customerElement));
        request.setParam(msgParam);

        TencentImGroupSendGroupMsgResponse response = tencentClientManage.getClient().execute(request);

        if (response.isSuccess()) {
            return true;
        }
        return false;
    }

    /**
     * 设置私聊功能
     *
     * @param account
     * @param privateChatTime
     * @param groupChatTime
     * @return
     */
    public boolean setNoSpeaking(String account, Integer privateChatTime, Integer groupChatTime) {
        TencentImSetNoSpeakingParam param = new TencentImSetNoSpeakingParam();
        param.setSetAccount(account);
        param.setC2cMsgNoSpeakingTime(privateChatTime);
        param.setGroupMsgNoSpeakingTime(groupChatTime);

        TencentImSetNoSpeakingRequest request = new TencentImSetNoSpeakingRequest();
        request.setParam(param);
        TencentImSetNoSpeakingResponse response = tencentClientManage.getClient().execute(request);
        return response.isSuccess();
    }

    public TencentImGetNoSpeakingResponse getNoSpeaking(String account) {
        TencentImGetNoSpeakingParam param = new TencentImGetNoSpeakingParam();
        param.setGetAccount(account);

        TencentImGetNoSpeakingRequest request = new TencentImGetNoSpeakingRequest();
        request.setParam(param);

        TencentImGetNoSpeakingResponse response = tencentClientManage.getClient().execute(request);
        return response;
    }

    //---以下为新版融云im的封装接口---

    /**
     * 创建token
     *
     * @param userId
     * @return
     */
    @Override
    public String buildToken(Long userId) {
        return null;
    }

    /**
     * 创建token
     *
     * @param imUserProfile
     * @return
     */
    @Override
    public String buildToken(ImUserProfile imUserProfile) {
        return null;
    }

    /**
     * 刷新用户资料
     *
     * @param imUserProfile
     * @return
     */
    @Override
    public boolean refreshUser(ImUserProfile imUserProfile) {
        return false;
    }

    /**
     * 发送私聊信息给用户
     *
     * @param userId
     * @param targetId
     * @param content
     * @return
     */
    @Override
    public boolean sendToPeer(Long userId, Long targetId, String content) {
        return false;
    }

    /**
     * 发送私聊信息给用户
     *
     * @param userId
     * @param targetId
     * @param data
     * @return
     */
    @Override
    public boolean sendToPeerCustomMsg(Long userId, Long targetId, String data) {
        return false;
    }

    /**
     * 发送私聊信息给用户
     *
     * @param userId
     * @param targetId
     * @param msg
     * @param pushContent
     * @param isIncludeSender
     * @return
     */
    @Override
    public boolean sendToPeerCustomMsg(Long userId, Long targetId, BaseMessage msg, String pushContent, Integer isIncludeSender) {
        return false;
    }

    /**
     * 发送私聊信息给用户
     *
     * @param userId
     * @param targetId
     * @param text
     * @param extra
     * @return
     */
    @Override
    public boolean sendToPeerTextMsg(Long userId, Long targetId, String text, String extra) {
        return false;
    }

    /**
     * 发送房间消息
     *
     * @param userId
     * @param roomId
     * @param content
     * @return
     */
    @Override
    public boolean sendToChatRoom(Long userId, String roomId, String content) {
        return false;
    }

    /**
     * 发送频道消息
     *
     * @param userId
     * @param roomId
     * @param content
     * @param lowPriority
     * @return
     */
    @Override
    public boolean sendToChannel(Long userId, Long roomId, String content, boolean lowPriority) {
        return false;
    }

    /**
     * 聊天室禁言
     *
     * @param roomId
     * @param userId
     * @param minutes
     * @return
     */
    @Override
    public boolean forbiddenChat(Long roomId, Long userId, Integer minutes) {
        return false;
    }

    /**
     * 解除禁言
     *
     * @param roomId
     * @param userId
     * @return
     */
    @Override
    public boolean enableChat(Long roomId, Long userId) {
        return false;
    }

    /**
     * 封禁用户
     *
     * @param roomId
     * @param userId
     * @param minutes
     * @return
     */
    @Override
    public boolean blockRoomUser(Long roomId, Long userId, Integer minutes) {
        return false;
    }

    /**
     * 解封用户
     *
     * @param roomId
     * @param userId
     * @return
     */
    @Override
    public boolean unblockRoomUser(Long roomId, Long userId) {
        return false;
    }

    /**
     * 设置聊天室属性
     *
     * @param roomId
     * @param key
     * @param value
     * @return
     */
    @Override
    public boolean setRoomProperty(Long roomId, String key, String value) {
        return false;
    }

    /**
     * 设置聊天室属性
     *
     * @param roomId
     * @param propertyMap
     */
    @Override
    public void setRoomProperties(Long roomId, Map<String, String> propertyMap) {

    }

    /**
     * 拉黑用户
     *
     * @param userId
     * @param targetId
     * @return
     */
    @Override
    public boolean addUserBlack(Long userId, Long targetId) {
        return false;
    }

    /**
     * 解除拉黑
     *
     * @param userId
     * @param targetId
     * @return
     */
    @Override
    public boolean removeUserBlack(Long userId, Long targetId) {
        return false;
    }

    /**
     * 获取拉黑名单
     *
     * @param userId
     * @return
     */
    @Override
    public List<String> getUserBlack(Long userId) {
        return null;
    }

    /**
     * 私聊禁言
     *
     * @param userId
     * @return
     */
    @Override
    public boolean forbiddenWords(Long userId) {
        return false;
    }

    /**
     * 取消私聊禁言
     *
     * @param userId
     * @return
     */
    @Override
    public boolean cancelForbiddenWords(Long userId) {
        return false;
    }

    /**
     * 获取融云历史消息
     *
     * @param date
     * @return
     */
    @Override
    public String getHistory(String date) {
        return null;
    }

    /**
     * 获取用户信息
     *
     * @param userId
     * @return
     */
    @Override
    public UserSimpleInfo getUserInfo(Long userId) {
        return null;
    }

    /**
     * 对比签名
     *
     * @param timestamp
     * @param nonce
     * @param sign
     * @return
     */
    @Override
    public boolean checkNotifySign(String timestamp, String nonce, String sign) {
        return false;
    }

    /**
     * 获取房间内在线用户id
     *
     * @param roomId
     * @return
     * @throws Exception
     */
    @Override
    public List<Long> onlineUserIdsInRoom(Long roomId) throws Exception {
        return null;
    }
}
