package com.starbuds.server.service.message.tencent;

import cn.hutool.core.util.RandomUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.service.message.tencent.constant.TencentApi;
import com.starbuds.server.service.message.util.HttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;

/**
 * 备注
 *
 * @author Lay
 * @date 2019/12/16
 */
@Slf4j
public abstract class AbstractTencentClient implements TencentClient {

    /**
     * 创建应用时即时通信 IM 控制台分配的 SDKAppID
     */
    private long sdkAppId;

    /**
     * 必须为 App 管理员帐号
     */
    private String identifier;

    /**
     * App 管理员帐号生成的签名
     */
    private String userSig;

    public AbstractTencentClient(@Value("${im.tencent.appKey}") long sdkAppId,
                                 @Value("${im.tencent.identifier}") String identifier,
                                 @Value("${im.tencent.userSig}") String userSig) {
        this.sdkAppId = sdkAppId;
        this.identifier = identifier;
        this.userSig = userSig;
    }

    /**
     * 执行请求
     *
     * @param request
     * @return
     */
    @Override
    public <T extends TencentResponse> T execute(TencentRequest<T> request) {
/*//        log.info("TencentIM 请求: {}", JSON.toJSONString(request));

        // 构造基本请求头
        HttpResponse response = HttpRequest.post(getApiUrl(request))
                .timeout(2000)
                .header(Header.CONTENT_TYPE, "application/json;charset=utf-8;")
                .body(JSON.toJSONString(request.getParam()))
                .execute();
        if (response.isOk()) {
            // 日志太多了，过滤掉
//            if (!TencentApi.CMD_APP_INFO.equals(request.getCommand())) {
//                log.info("TencentIM 请求结果: {}", response.body());
//            }
            return JSON.parseObject(response.body(), request.getResponseClass());
        }
        return null;*/

        try {
            return JSON.parseObject(HttpUtil.post(getApiUrl(request), JSON.toJSONString(request.getParam())), request.getResponseClass());
        } catch (Exception e) {
            log.error("腾讯云请求失败", e);
        }
        return null;
    }

    private String getApiUrl(TencentRequest request) {
        return String.format("%s/%s/%s/%s?sdkappid=%d&identifier=%s&usersig=%s&random=%d&contenttype=json",
                TencentApi.GATEWAY,
                request.getVersion(),
                request.getServiceName(),
                request.getCommand(),
                sdkAppId,
                identifier,
                userSig,
                RandomUtil.randomLong(0L, 4294967295L));
    }
}
