package com.starbuds.server.service.message.util;

import lombok.extern.slf4j.Slf4j;
import okhttp3.*;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * @author aoe
 * @date 2021/1/7
 */
@Slf4j
public class HttpUtil {

    private static OkHttpClient client;

    private static final MediaType MEDIA_TYPE_JSON = MediaType.get("application/json; charset=utf-8");

    static {
        client = new OkHttpClient();
    }

    private HttpUtil() {
    }

    public static String post(String url, String json) {
        return post(url, json, null);
    }

    public static String post(String url, String json, Map<String, String> headers) {
        if (headers == null) {
            headers = new HashMap<>();
        }

        RequestBody body = RequestBody.create(json, MEDIA_TYPE_JSON);
        Request request = new Request.Builder()
                .url(url)
                .post(body)
                .headers(Headers.of(headers))
                .build();
        try (Response response = client.newCall(request).execute()) {
            if (!response.isSuccessful()) {
                log.warn("Request Failed[{}] {} {}", response.code(), url, json);
            }
            return response.body().string();
        } catch (Exception e) {
            log.error(String.format("Request Failed url = %s body = %s Headers = %s", url, json, headers), e);
        }
        return null;
    }

    public static String get(String url) throws IOException {
        Request request = new Request.Builder()
                .url(url)
                .get()
                .build();
        try (Response response = client.newCall(request).execute()) {
            return response.body().string();
        }
    }
}
