package com.starbuds.repair;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.service.message.util.HttpUtil;
import com.starbuds.test.util.FileTools;
import io.rong.RongCloud;
import io.rong.models.chatroom.ChatroomModel;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.dubbo.common.utils.CollectionUtils;
import org.junit.Assert;
import org.junit.jupiter.api.Test;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 手动清理语音房在线人数
 *
 * @author aoe
 * @date 2021/9/30
 */
@Slf4j
public class RoomHeat {

    private static RongCloud rongCloud = RongCloud.getInstance("e0x9wycfevs5q", "pMsSuJnPr5V");

    @Test
    public void t() {
        System.out.println("222");
    }

    /**
     * SELECT roomId, roomName, roomHeat
     * FROM tb_rtc_room
     * ORDER BY roomHeat DESC
     * LIMIT 0,100
     *
     * @throws Exception
     */
    @Test
    public void removeUsers() throws Exception {
        // 所有房间id
        List<Long> roomIds = roomIds();
        List<String> delCommands = new ArrayList<>();
        int i = 1;
        for (Long roomId : roomIds) {
            List<Long> redisUserIds = redisUserIds(roomId);
            List<Long> onLineUserIds = onlineUserIdsInRoom(roomId);

            if (CollectionUtils.isEmpty(redisUserIds) && CollectionUtils.isEmpty(onLineUserIds)) {
                log.info("房间内没有人 RoomId = {}", roomId);
                continue;
            }

            List<Long> removeUserIds = redisUserIds.stream()
                    // 不在线的用户
                    .filter(redisUserId -> {
                                long onlineCount = onLineUserIds.stream()
                                        .filter(onlineUserId -> onlineUserId - redisUserId == 0)
                                        .count();
                                return onlineCount < 1;
                            }
                    )
                    .collect(Collectors.toList());
            if (CollectionUtils.isEmpty(removeUserIds)) {
                log.info("房间内没有要清理的用户 RoomId = {}", roomId);
                continue;
            }

            long count = removeUserIds.stream().filter(id -> id - 271581569875968L == 0).count();
            // 不能删除在线用户
            Assert.assertEquals(0, count);

            String key = RedisRtcKeys.getRtcRoomUser(roomId);
            List<String> idArr = removeUserIds.stream().map(id -> "'" + id + "'").collect(Collectors.toList());
            String delCmd = String.format("redisHashDel('%s', [%s]);", key, StringUtils.join(idArr, ","));
            delCommands.add(delCmd);
        }

        FileTools.writeToFile("/Users/aoe/IdeaProjects/paqu/fastFish/service-message/src/test/resources/temp/HDEL.txt", delCommands);

    }

    // 10069 10087

    /**
     * 获得房间内在线用户id（最多返回 500 个成员信息）
     * 获取聊天室成员 https://数据中心域名/chatroom/user/query.json
     * https://doc.rongcloud.cn/imserver/server/v1/chatroom/info#user_query
     *
     * @param roomId
     * @return 房间内在线用户id
     * @throws Exception
     */
    private static List<Long> onlineUserIdsInRoom(Long roomId) throws Exception {
        if (roomId == null) {
            return Collections.emptyList();
        }

        ChatroomModel chatroomModel = new ChatroomModel()
                .setId(String.valueOf(roomId))
                .setCount(500)
                .setOrder(2);
        return rongCloud.chatroom.get(chatroomModel).getMembers().stream()
                .map(user -> NumberUtils.toLong(user.id))
                .distinct()
                .collect(Collectors.toList());
    }

    /**
     * 所有房间id
     *
     * @return
     */
    private static List<Long> roomIds() {
        List<String> ids = FileTools.readList("/Users/aoe/IdeaProjects/paqu/fastFish/service-message/src/test/resources/temp/roomIds.txt");
        return ids.stream()
                .map(NumberUtils::toLong)
                .collect(Collectors.toList());
    }

    /**
     * Redis 中存储的 用户id
     *
     * @param roomId
     * @return
     * @throws IOException
     */
    private static List<Long> redisUserIds(Long roomId) throws IOException {
        String url = "http://redis.manage.ti1michat.com/query/hashGetAll?key=rtc:room:user:" + roomId;
//        String rsp = HttpUtil.get("http://redis.manage.timichat.com/query/zRangeWithScores?key=rtc:room:online:" + roomId);
//        log.info(LogUtil.devLog(url));
        String rsp = HttpUtil.get(url);
        JSONObject json = JSON.parseObject(rsp);
        return json.keySet().stream()
                .map(key -> NumberUtils.toLong(key))
                .distinct()
                .collect(Collectors.toList());
    }
}
